(************************************************************************
 *                                                                      *
 *   Ant Movie Catalog 3.x                                              *
 *   (C) 2000-2011 Antoine Potten, Mickal Vanneufville                 *
 *   http://www.antp.be/software                                        *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************)

unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ActnList, ImgList, Menus, StdCtrls, Buttons, Clipbrd, ComCtrls,
  StdActns, Contnrs,

  ElTree, ElList, ElHeader, TB2Common, TB2Item, TB2MRU, TB2Toolbar, TB2Dock,
  TB2ToolWindow, TBX, TBXExtItems, TBXSwitcher, TBXStatusBars,

  AntJvDragDrop, AntStringList, AntCorelButton,

  framemovie, MovieClass, FileManager, ConstValues,
  FramePictureSelectionOptions,

  HTMLSubs, HtmlView, MetaFilePrinter, PrintStatusForm, PreviewForm,

  JPEG, PNGImage, AntJvGIF, rkIntegerList, rkSmartView;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

const
  strDefaultCaption = 'Ant Movie Catalog Unofficial %s - %s';
  strDefaultTask = '%s - Ant Movie Catalog Unofficial';
  CM_UpdateView = WM_USER + 2102; // Custom Message...

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

type
  TJpegImageFix = class(TJpegImage)
  protected
    procedure Draw(ACanvas: TCanvas; const Rect: TRect); Override;
  end;

  {ThumbsViewer}
  PCacheItem = ^TCacheItem;
  TCacheItem = record
    Idx: Integer;
    Size: Integer;
    Age: TDateTime;
    Scale: Integer;
    Bmp: TBitmap;
  end;

  ThumbThread = class(TThread)
  private
    { Private declarations }
    ThumbsViewer: TrkSmartView;
    ListView1: TElTree;
    MovieList: TMovieList;

  protected
    procedure Execute; override;
  public
    constructor Create(Thumbs: TrkSmartView; Items: TElTree; Movies: TMovieList);
  end;
  {End ThumbsViewer}
  
  TMainWindow = class(TForm)
    ActionDisplayMainToolbar: TAction;
    ActionDisplayPictureToolbar: TAction;
    ActionDisplayStatusBar: TAction;
    ActionExit: TAction;
    ActionFileExport: TAction;
    ActionFileImport: TAction;
    ActionFileNew: TAction;
    ActionFileOpen: TAction;
    ActionFileOpenNoRecent: TAction;
    ActionFilePrint: TAction;
    ActionFileProperties: TAction;
    ActionFileSave: TAction;
    ActionFileSaveAs: TAction;
    ActionFindDisplay: TAction;
    ActionFindFindnext: TAction;
    ActionFindWholefield: TAction;
    ActionGroupNone: TAction;
    ActionHelpAbout: TAction;
    ActionHelpIndex: TAction;
    ActionLanguage: TAction;
    ActionList1: TActionList;
    ActionLoan: TAction;
    ActionMenuFile: TAction;
    ActionMenuGet: TAction;
    ActionMenuGroup: TAction;
    ActionMenuSort: TAction;
    ActionMenuHelp: TAction;
    ActionMenuMovie: TAction;
    ActionMenuPicture: TAction;
    ActionMenuTools: TAction;
    ActionMovieAdd: TAction;
    ActionMovieCheck: TAction;
    ActionMovieCopy: TAction;
    ActionMovieDelete: TAction;
    ActionMovieFind: TAction;
    ActionMovieImportCD: TAction;
    ActionMovieImportFiles: TAction;
    ActionMovieImportScript: TAction;
    ActionMovieNext: TAction;
    ActionMovieNumber: TAction;
    ActionMoviePaste: TAction;
    ActionMoviePictureShow: TAction;
    ActionMoviePrevious: TAction;
    ActionMovieRenumber: TAction;
    ActionMovieSearch: TAction;
    ActionMovieSelCheck: TAction;
    ActionMovieSelGroup: TAction;
    ActionMovieSelUncheck: TAction;
    ActionMovieStats: TAction;
    ActionMovieUncheck: TAction;
    ActionMovieUndo: TAction;
    ActionOptions: TAction;
    ActionPicCopy: TAction;
    ActionPicDelete: TAction;
    ActionPicSaveAs: TAction;
    ActionPicSelect: TAction;
    ActionPicUndock: TAction;
    ActionRefresh: TAction;
    ActionToolsGrid: TAction;
    ActionToolsScripting: TAction;
    ActionURLBrowse: TAction;
    ActionURLOpen: TAction;
    BtnFindNext: TCorelButton;
    CBDisplayResults: TCheckBox;
    CBWholefield: TCheckBox;
    cbxField: TComboBox;
    DockBottomList: TTBXDock;
    DockImageLeft: TTBXDock;
    DockImageTop: TTBXDock;
    DockMainBottom: TTBXDock;
    DockMainLeft: TTBXDock;
    DockMainTop: TTBXDock;
    DockRightInfo: TTBXDock;
    EValue: TEdit;
    Fields: TAntStringList;
    FrmMovie: TMovieFrame;
    ImageListHot: TImageList;
    ImageListNormal: TTBImageList;
    LField: TLabel;
    ListView1: TElTree;
    LValue: TLabel;
    Messages: TAntStringList;
    MnuFil__1: TTBXSeparatorItem;
    MnuFil__2: TTBXSeparatorItem;
    MnuFil__3: TTBXSeparatorItem;
    MnuFilExp: TTBXItem;
    MnuFilImp: TTBXItem;
    MnuFilNew: TTBXItem;
    MnuFilOpn: TTBXItem;
    MnuFilPrn: TTBXItem;
    MnuFilPrp: TTBXItem;
    MnuFilSaa: TTBXItem;
    MnuFilSav: TTBXItem;
    MnuFilXit: TTBXItem;
    MnuGetFil: TTBXItem;
    MnuGetScr: TTBXItem;
    MnuGrp__1: TTBXSeparatorItem;
    MnuGrpNon: TTBXItem;
    MnuHlpAbt: TTBXItem;
    MnuHlpIdx: TTBXItem;
    MnuMlp__1: TTBXSeparatorItem;
    MnuMlp__2: TTBXSeparatorItem;
    MnuMlp__3: TTBXSeparatorItem;
    MnuMlpAdd: TTBXItem;
    MnuMlpChk: TTBXItem;
    MnuMlpCpy: TTBXItem;
    MnuMlpDel: TTBXItem;
    MnuMlpGet: TTBXSubmenuItem;
    MnuMlpGrd: TTBXItem;
    MnuMlpGrp: TTBXSubmenuItem;
    MnuMlpNum: TTBXItem;
    MnuMlpPst: TTBXItem;
    MnuMlpSlc: TTBXItem;
    MnuMlpSlg: TTBXItem;
    MnuMlpSlu: TTBXItem;
    MnuMlpUch: TTBXItem;
    MnuMnuFil: TTBXSubmenuItem;
    MnuMnuHlp: TTBXSubmenuItem;
    MnuMnuMov: TTBXSubmenuItem;
    MnuMnuTls: TTBXSubmenuItem;
    MnuMov__1: TTBXSeparatorItem;
    MnuMov__2: TTBXSeparatorItem;
    MnuMovAdd: TTBXItem;
    MnuMovCpy: TTBXItem;
    MnuMovDel: TTBXItem;
    MnuMovFnd: TTBXItem;
    MnuMovGet: TTBXSubmenuItem;
    MnuMovNum: TTBXItem;
    MnuMovPic: TTBXSubmenuItem;
    MnuMovPst: TTBXItem;
    MnuMovSch: TTBXSubmenuItem;
    MnuMovUnd: TTBXItem;
    MnuMpi__1: TTBXSeparatorItem;
    MnuMpiCpy: TTBXItem;
    MnuMpiDck: TTBXItem;
    MnuMpiDel: TTBXItem;
    MnuMpiDis: TTBXItem;
    MnuMpiLoa: TTBXItem;
    MnuMpiSav: TTBXItem;
    MnuPicCpy: TTBXItem;
    MnuPicDck: TTBXItem;
    MnuPicDel: TTBXItem;
    MnuPicLoa: TTBXItem;
    MnuPicSav: TTBXItem;
    MnuTbp__1: TTBXSeparatorItem;
    MnuTbpMtb: TTBXItem;
    MnuTbpOpt: TTBXItem;
    MnuTbpPtb: TTBXItem;
    MnuTbpStb: TTBXItem;
    MnuTls__1: TTBXSeparatorItem;
    MnuTls__2: TTBXSeparatorItem;
    MnuTlsGrd: TTBXItem;
    MnuTlsGrp: TTBXSubmenuItem;
    MnuTlsSort: TTBXSubmenuItem;
    MnuTlsLng: TTBXItem;
    MnuTlsLoa: TTBXItem;
    MnuTlsOpt: TTBXItem;
    MnuTlsRen: TTBXItem;
    MnuTlsScr: TTBXItem;
    MnuTlsSta: TTBXItem;
    MnuUrpBrw: TTBXItem;
    MnuUrpOpen: TTBXItem;
    MoviePicture: TImage;
    PanelLeft: TPanel;
    PanelMovieInfos: TPanel;
    PanelPicture: TPanel;
    PopupEURL: TTBXPopupMenu;
    PopupMovieList: TTBXPopupMenu;
    PopupToolbar: TTBXPopupMenu;
    ScrollBox1: TScrollBox;
    SplitterBottomList: TSplitter;
    SplitterMovieInfos: TSplitter;
    SplitterRightInfo: TSplitter;
    StatusBar1: TTBXStatusBar;
    TBItemAbout: TTBXItem;
    TBItemExit: TTBXItem;
    TBItemFileExport: TTBXItem;
    TBItemFileImport: TTBXItem;
    TBItemFileNew: TTBXItem;
    TBItemFileOpen: TTBXItem;
    TBItemFileOpenMRU: TTBXSubmenuItem;
    TBItemFileOpenNone: TTBXItem;
    TBItemFilePrint: TTBXItem;
    TBItemFileProperties: TTBXItem;
    TBItemFileSave: TTBXItem;
    TBItemFileSaveAs: TTBXItem;
    TBItemGetInfo: TTBXSubmenuItem;
    TBItemGetInfoFiles: TTBXItem;
    TBItemGetInfoScript: TTBXItem;
    TBItemGrid: TTBXItem;
    TBItemGroup: TTBXSubmenuItem;
    TBItemHelp: TTBXItem;
    TBItemLoans: TTBXItem;
    TBItemMovieAdd: TTBXItem;
    TBItemMovieCopy: TTBXItem;
    TBItemMovieDelete: TTBXItem;
    TBItemMovieFind: TTBXItem;
    TBItemMovieNumber: TTBXItem;
    TBItemMoviePaste: TTBXItem;
    TBItemMoviePicture: TTBXItem;
    TBItemMoviePictureMenu: TTBXSubmenuItem;
    TBItemMovieUndo: TTBXItem;
    TBItemPreferences: TTBXItem;
    TBItemRenumber: TTBXItem;
    TBItemScripting: TTBXItem;
    TBItemSearch: TTBXSubmenuItem;
    TBItemStats: TTBXItem;
    TBMRUList1: TTBXMRUList;
    TBMRUListItem1: TTBXMRUListItem;
    TBMRUListItem2: TTBXMRUListItem;
    TBSeparatorFile: TTBXSeparatorItem;
    TBSeparatorGet: TTBXSeparatorItem;
    TBSeparatorHelp: TTBXSeparatorItem;
    TBSeparatorMisc: TTBXSeparatorItem;
    TBSeparatorMovie: TTBXSeparatorItem;
    TBSeparatorMovieEdit: TTBXSeparatorItem;
    TBXSwitcher1: TTBXSwitcher;
    ToolbarFind: TTBXToolWindow;
    ToolbarMain: TTBXToolbar;
    ToolbarMenu: TTBXToolbar;
    ToolbarPicture: TTBXToolbar;
    ToolbarPictureWindow: TTBXToolWindow;
    ActionHelpVersion: TAction;
    MnuHlpVer: TTBXItem;
    TBItemLanguage: TTBXItem;
    TBItemRandom: TTBXItem;
    ActionMovieRandom: TAction;
    TBItemStretchList: TTBXItem;
    ActionStretchList: TAction;
    MnuTlsStr: TTBXItem;
    MnuMovRan: TTBXItem;
    HTMLViewer: THTMLViewer;
    ActionDisplayHTML: TAction;
    TBItemDisplayHTML: TTBXItem;
    MnuTlsHTM: TTBXItem;
    ThumbsPanel: TPanel;
    ThumbsPanelBottom: TPanel;
    ThumbsSizer: TTrackBar;
    ProgressBar1: TProgressBar;
    ThumbsViewer: TrkSmartView;
    ActionDisplayThumbnails: TAction;
    MnuTlsThu: TTBXItem;
    TBItemDisplayThumbnails: TTBXItem;
    TBItemSort: TTBXSubmenuItem;
    MnuMlpSort: TTBXSubmenuItem;
    MnuTlsSortAscend: TTBXItem;
    MnuTlsSortDescend: TTBXItem;
    MnuTlsSortSep: TTBXSeparatorItem;
    ActionSortAscend: TAction;
    ActionSortDescend: TAction;
    procedure ActionDisplayMainToolbarExecute(Sender: TObject);
    procedure ActionDisplayPictureToolbarExecute(Sender: TObject);
    procedure ActionDisplayStatusBarExecute(Sender: TObject);
    procedure ActionExitExecute(Sender: TObject);
    procedure ActionFileExportExecute(Sender: TObject);
    procedure ActionFileImportExecute(Sender: TObject);
    procedure ActionFileNewExecute(Sender: TObject);
    procedure ActionFileOpenExecute(Sender: TObject);
    procedure ActionFilePrintExecute(Sender: TObject);
    procedure ActionFilePropertiesExecute(Sender: TObject);
    procedure ActionFileSaveAsExecute(Sender: TObject);
    procedure ActionFileSaveExecute(Sender: TObject);
    procedure ActionFindDisplayExecute(Sender: TObject);
    procedure ActionFindFindnextExecute(Sender: TObject);
    procedure ActionFindWholefieldExecute(Sender: TObject);
    procedure ActionGroupNoneExecute(Sender: TObject);
    procedure ActionSortExecute(Sender: TObject);
    procedure ActionHelpAboutExecute(Sender: TObject);
    procedure ActionHelpIndexExecute(Sender: TObject);
    procedure ActionLanguageExecute(Sender: TObject);
    procedure ActionLoanExecute(Sender: TObject);
    procedure ActionMenuFileExecute(Sender: TObject);
    procedure ActionMenuGetExecute(Sender: TObject);
    procedure ActionMenuGroupExecute(Sender: TObject);
    procedure ActionMenuSortExecute(Sender: TObject);
    procedure ActionMenuHelpExecute(Sender: TObject);
    procedure ActionMenuMovieExecute(Sender: TObject);
    procedure ActionMenuPictureExecute(Sender: TObject);
    procedure ActionMenuToolsExecute(Sender: TObject);
    procedure ActionMovieAddExecute(Sender: TObject);
    procedure ActionMovieCheckExecute(Sender: TObject);
    procedure ActionMovieCopyExecute(Sender: TObject);
    procedure ActionMovieDeleteExecute(Sender: TObject);
    procedure ActionMovieFindExecute(Sender: TObject);
    procedure ActionMovieImportCDExecute(Sender: TObject);
    procedure ActionMovieImportFilesExecute(Sender: TObject);
    procedure ActionMovieImportScriptExecute(Sender: TObject);
    procedure ActionMovieNextExecute(Sender: TObject);
    procedure ActionMovieNumberExecute(Sender: TObject);
    procedure ActionMoviePasteExecute(Sender: TObject);
    procedure ActionMoviePictureShowExecute(Sender: TObject);
    procedure ActionMoviePreviousExecute(Sender: TObject);
    procedure ActionMovieRenumberExecute(Sender: TObject);
    procedure ActionMovieSearchExecute(Sender: TObject);
    procedure ActionMovieSelCheckExecute(Sender: TObject);
    procedure ActionMovieSelGroupExecute(Sender: TObject);
    procedure ActionMovieSelUncheckExecute(Sender: TObject);
    procedure ActionMovieStatsExecute(Sender: TObject);
    procedure ActionMovieUncheckExecute(Sender: TObject);
    procedure ActionMovieUndoExecute(Sender: TObject);
	  procedure ActionMovieRandomExecute(Sender: TObject);
    procedure ActionOptionsExecute(Sender: TObject);
    procedure ActionPicCopyExecute(Sender: TObject);
    procedure ActionPicDeleteExecute(Sender: TObject);
    procedure ActionPicSaveAsExecute(Sender: TObject);
    procedure ActionPicSelectExecute(Sender: TObject);
    procedure ActionPicUndockExecute(Sender: TObject);
    procedure ActionRefreshExecute(Sender: TObject);
    procedure ActionToolsGridExecute(Sender: TObject);
    procedure ActionToolsScriptingExecute(Sender: TObject);
    procedure ActionURLBrowseExecute(Sender: TObject);
    procedure ActionURLOpenExecute(Sender: TObject);
    procedure ActionHelpVersionExecute(Sender: TObject);
    procedure ActionStretchListExecute(Sender: TObject);
    procedure ActionDisplayHTMLExecute(Sender: TObject);
    procedure ActionDisplayThumbnailsExecute(Sender: TObject);
    procedure cbxFieldChange(Sender: TObject);
    procedure DockBottomListRequestDock(Sender: TObject; Bar: TTBCustomDockableWindow; var Accept: Boolean);
    procedure DockImageLeftRequestDock(Sender: TObject; Bar: TTBCustomDockableWindow; var Accept: Boolean);
    procedure DockMainBottomRequestDock(Sender: TObject; Bar: TTBCustomDockableWindow; var Accept: Boolean);
    procedure DockMainLeftRequestDock(Sender: TObject; Bar: TTBCustomDockableWindow; var Accept: Boolean);
    procedure DockMainTopRequestDock(Sender: TObject; Bar: TTBCustomDockableWindow; var Accept: Boolean);
    procedure DragDropFilesDrop(Sender: TObject; Pos: TPoint; Value: TStringList);
    procedure MovieFieldKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure EValueChange(Sender: TObject);
    procedure EValueKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FrmMovieEURLButtonClick(Sender: TObject);
    procedure ListView1AfterSelectionChange(Sender: TObject);
    procedure ListView1CompareItems(Sender: TObject; Item1, Item2: TElTreeItem; var res: Integer);
    procedure ListView1ItemChecked(Sender: TObject; Item: TElTreeItem);
    procedure ListView1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListView1Resize(Sender: TObject);
    procedure MoviePictureClick(Sender: TObject);
    procedure PopupMovieListPopup(Sender: TObject);
    procedure SplitterBottomListCanResize(Sender: TObject; var NewSize: Integer; var Accept: Boolean);
    procedure TBMRUList1Click(Sender: TObject; const Filename: String);
    procedure ToolbarFindVisibleChanged(Sender: TObject);
    procedure ToolbarMainClose(Sender: TObject);
    procedure ToolbarPictureClose(Sender: TObject);
    procedure ToolbarPictureWindowClose(Sender: TObject);
    procedure ToolbarPictureWindowDockChanged(Sender: TObject);
    procedure ListView1HeaderColumnClick(Sender: TObject; SectionIndex: Integer);
    procedure MovieFieldKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FrmMovieMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure HTMLViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure HTMLViewerImageRequest(Sender: TObject; const SRC: String; var Stream: TMemoryStream);
    procedure HTMLViewerImageClick(Sender, Obj: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure HTMLViewerKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    {ThumbsViewer}
    procedure CMUpdateView(var message: TMessage); message CM_UpdateView;
    procedure StartThumbThread;
    procedure StopThumbThread;
    procedure ThumbsViewerStop;
    procedure ThumbsViewerStart;
    procedure GenCellColors;
    procedure ItemPaintBasic(Canvas: TCanvas; R: TRect; State: TsvItemState);
    procedure ThumbsViewerCellPaint(Sender: TObject; Canvas: TCanvas; Cell: TRect;
      IdxA, Idx: Integer; State: TsvItemState);
    procedure ThumbsViewerSelecting(Sender: TObject; Count: Integer);
    procedure ThumbsSizerChange(Sender: TObject);
    procedure ThumbsViewerMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ThumbsViewerDividerPaint(Sender: TObject; Canvas: TCanvas;
      Cell: TRect; Group: PSmartGroup; State: TsvItemState);
    procedure ThumbsViewerResize(Sender: TObject);
    procedure ThumbsViewerHeaderPaint(Sender: TObject; Canvas: TCanvas;
      Header: TRect; Offset, Active: Integer; State: TsvItemState;
      Columns: array of Integer);
    procedure ThumbsViewerHeaderClick(Sender: TObject; Column: Integer);
    procedure ListView1ItemCollapse(Sender: TObject; Item: TElTreeItem);
    procedure ListView1ItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure ThumbsViewerMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ThumbsViewerDblClick(Sender: TObject);
    procedure ThumbsViewerKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    {End ThumbsViewer}

  private
    ColumnTitleDisplay: Integer;
    UsePrefixes:        Boolean;
    MovieList:          TMovieList;
    MovieSave:          TMovie;
    PanelLeftOldWidth:  Integer;
    FPreviousItem:      TElTreeItem;
    FCatalogFile:       TFileManager;
    FGroupField:        Integer;
    FSortField:         Integer;
    DragDropFiles:      TJvDragDrop;
    MoviesVisible:      Integer;
    GroupItemEmpty:     TElTreeItem;
    MStream:            TMemoryStream;
    MStreamAppr:        array[0..4] of TMemoryStream;
    MStreamAppr10:      array[0..10] of TMemoryStream;
    HTMLTemplatePreGenerated: TStringList;
    HTMLTemplatePath:   string;
    {ThumbsViewer}
    ThumbSizeW, ThumbSizeH: Integer;
    ThumbJPEG: TJpegImageFix;
    ThumbGIF: TJvGIFImage;
    ThumbPNG: TPNGObject;
    {End ThumbsViewer}
    procedure ListView1UpdateItem;
    procedure HTMLViewerUpdate;
    procedure ListView1EnsureVisiblePlus;
    procedure SearchInternetClick(Sender: TObject);
    function GetTitle: string;
    procedure SetTitle;
    procedure SetStatus;
    procedure UpdateMRU(const strFilePath: string);
    procedure SetGroupField(Field: Integer);
    procedure SetSortField(Field: Integer; Descend: Boolean);
    procedure OnFileModified(Sender: TObject; State: Boolean);
    procedure OnFileChange(Sender: TObject; AFileName: TFileName; var Result: Boolean);
    procedure OnNewFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
    procedure OnOpenFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
    procedure OnSaveFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
    procedure OnDialogTypeChange(Sender: TObject);
    procedure SetSaveFilter;
    procedure NewItem;
    function OpenItem(const Item: TElTreeItem): integer; overload;
    function OpenItem(const AMovie: TMovie): integer; overload;
    function SaveCurrentItem: integer; overload;
    function SaveCurrentItem(const AMovie: TMovie): integer; overload;
    function AddItem(Movie: TMovie; GroupItem: TElTreeItem): TElTreeItem;
    {procedure ModifyItem(Item: TElTreeItem; MovieTmp: TMovie);}
    function DeleteItem(Movie: TMovie; const CanShowWindow, ShowAllButtons: Boolean): Integer;
    procedure UpdateItem(Item : TElTreeItem);
    function AddGroupItem(GroupName: string) : TElTreeItem;
    function GetGroupNameWithoutCount(GroupName: string) : string;
    procedure UpdateGroupName(GroupItem: TElTreeItem; GroupNameWithoutCount: string);
    procedure ClearItem;
    procedure OnMovieFieldChange(Sender: TObject);
    procedure LoadMoviePicture;
    procedure MovieSelected;
    procedure RefreshMovieList(const KeepSelection: Boolean = True);
    procedure StoreSelectedState;
    procedure LoadSelectedState;
    procedure LoadOptions;
    procedure ApplyOptions;
    procedure ApplyLanguage;
    procedure RebuildSearchMenu;
    procedure SaveOptions;
    procedure LoadLists;
    procedure SaveLists;
    procedure UpdateLists;
    procedure ImportMoviePicture(const AFileName: TFileName; const ImportMethod: TPictureSelectOption);
    function OnAppHelp(Command: Word; Data: Longint; var CallHelp: Boolean): Boolean;
    procedure PreGenerateHTMLTemplate;
    function GenerateHTMLSelectedMovie : string;
    {ThumbsViewer}
    procedure ThumbsGetThumbnail(Sender: TObject; Movie: TMovie);
    function ThumbBmp(idx: Integer): TBitmap;
    procedure SetThumbSize(Value: Integer; UpdateTrackbar: Boolean);
    procedure ClearThumbs;
    procedure ClearThumbsPool;
    {End ThumbsViewer}

  protected
    {ThumbsViewer}
    ThumbThr: ThumbThread;
    ThreadDone: Boolean;
    {End ThumbsViewer}

  public
    {ThumbsViewer}
    // Colors
    cGSelectedStart,
    cGSelectedEnd,
    cGHotStart,
    cGHotEnd,
    cGDisabledStart,
    cGDisabledEnd,
    cGHeaderStart,
    cGHeaderEnd,
    cGHeaderHotStart,
    cGHeaderHotEnd,
    cGHeaderSelStart,
    cGHeaderSelEnd,
    cHot,
    cSelected,
    cDisabled,
    cBackground,
    cLineHighLight: TColor;
    cShadeSelect: TColor;
    cShadeDisabled: TColor;
    CellShade0: TColor;
    CellShade1: TColor;
    CellShade2: TColor;
    CellShade3: TColor;
    CellShade4: TColor;
    CellBkgColor: TColor;
    CellBrdColor: array[Boolean, Boolean] of TColor;
    {End ThumbsViewer}

    function ExecuteAction(Action: TBasicAction): Boolean; override;
  end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

var
  MainWindow: TMainWindow;
  {ThumbsViewer}
  CellJpeg : TJpegImageFix;
  WI, HI, TW, TH, HSX: Integer;
  CellScale: Integer;
  CellStyle: Integer;
  ThumbsPool: TList;
  PoolSize, MaxPool: Integer;
  {End ThumbsViewer}

implementation

uses
  CommDlg, ExtDlgs, Math, DateUtils, StrUtils,

  IdURI, AntJvSpin,

  BrowseForFolderU, rmkGradient, rmkFunctions,

  {$IFNDEF DISABLETHEMES}
  TBXOfficeXPTheme,
  {$ENDIF}

  functions_str, functions_sys, functions_video, functions_files, functions_gui,
  functions_xml,

  Global, options, about, export, number, loan, splash, stats,
  renumber, fields, properties, loanhistory, getscript, Languageselect,
  ProgramSettings, printform, PictureSelection, PictureDragDrop, pictureform,
  progress, getfile, import2, Variants, SyncObjs;

{$R *.DFM}

const
  // Messages
  msgDeleteMovie      = 0;
  msgUndo             = 1;
  msgStatusCount      = 2;
  msgStatusVisible    = 3;
  msgUnsavedFile      = 4;
  msgDoNotConfirm     = 5;
  msgUnsavedStatus    = 6;
  msgPicture          = 7;
  msgPicSaveAs        = 8;
  msgImporting        = 9;
  msgSaving           = 10;
  msgLoading          = 11;
  msgPicOpen          = 12;
  msgPicDelete        = 13;
  msgPicLoadFailed    = 14;
  msgByteString       = 15;
  msgSelectFilesInfo  = 16;
  msgSearchInfo       = 17;
  msgNoRecentFile     = 18;
  msgXmlConvertImages = 19;
  msgGroupBy          = 20;
  msgGroupEmpty       = 21;
  msgPicDeleteLink    = 22;
  msgPicReplaceLink   = 23;
  msgPicCopyNoCatPath = 24;
  msgPicHintStored    = 25;
  msgPicHintLinked    = 26;
  msgPicHintInfo      = 27;
  msgPrintLoading     = 28;
  msgPrintSearching   = 29;
  msgSelectFileURL    = 30;
  msgGroupUnique      = 31;
  msgFindFieldAll     = 32;
  msgSortBy           = 33;

  panelHint     = 0;
  panelModified = 1;
  panelCount    = 2;

{-------------------------------------------------------------------------------
   Update components when selected item informations changes
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1UpdateItem;
begin
  if (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil) and
    (ListView1.Selected.Data <> nil) and FrmMovie.Modified then
  begin
    if(MovieSave = nil) then
    begin
      MovieSave := TMovie.Create;
      TMovie(ListView1.Selected.Data).Lock;
      MovieSave.Assign(ListView1.Selected.Data);
      TMovie(ListView1.Selected.Data).UnLock;
    end;
    FrmMovie.SaveToObject(ListView1.Selected.Data);
    UpdateItem(ListView1.Selected);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.HTMLViewerUpdate;
begin
  if ActionDisplayHTML.Checked then
  begin
    HTMLViewer.VScrollBarPosition := 0;
    HTMLViewer.LoadFromString(GenerateHTMLSelectedMovie, HTMLTemplatePath);
  end;
end;

{-------------------------------------------------------------------------------
  Ensure Visible (Plus for groups) on selected item
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1EnsureVisiblePlus;
begin
  with ListView1 do
  begin
    if (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil) then
    begin
      EnsureVisible(ListView1.Selected);
      if ListView1.Selected.Parent <> nil then
      begin
        EnsureVisible(ListView1.Selected.Parent);
        EnsureVisibleBottom(ListView1.Selected);
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
   Item management
-------------------------------------------------------------------------------}

procedure TMainWindow.ClearItem;
begin
  FrmMovie.LoadFromObject(Settings.rOptions.rMovieInformation.rDefaultMovie.Values);
  HTMLViewerUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.NewItem;
var
  Item: TElTreeItem;
  NewMovie, Existing: TMovie;
begin
  SaveCurrentItem;
  //ListView1UpdateItem; <-- Check in SaveCurrentItem

  ListView1.DeselectAll;
  MovieSelected;

  NewMovie := MovieList.Add;
  NewMovie.InitFields;
  NewMovie.Assign(Settings.rOptions.rMovieInformation.rDefaultMovie.Values);
  NewMovie.iNumber := NumberWin.ENumber.AsInteger;
  NewMovie.bChecked := True;

  case NumberWin.grpNotUnique.ItemIndex of
    1:
      begin
        Existing := MovieList.Find(NumberWin.ENumber.AsInteger);
        if Existing <> nil then
        begin
          if Settings.rOptions.rMovieInformation.FirstAvailable then
            Existing.iNumber := MovieList.FirstFreeNumber
          else
            Existing.iNumber := MovieList.MaxNumber;
        end;
        RefreshMovieList(False);
      end;
    2:
      begin
        MovieList.ShiftNumbers(NumberWin.ENumber.AsInteger, NewMovie);
        RefreshMovieList(False);
      end;
  end;

  ListView1.Items.BeginUpdate;
  if (FGroupField <> -1) and Settings.rOptions.rMovieList.GroupEmpty then
  begin
    if GroupItemEmpty = nil then
    begin
      GroupItemEmpty := AddGroupItem(Messages.Strings[msgGroupEmpty]);
      Item := AddItem(NewMovie, GroupItemEmpty);
      UpdateGroupName(GroupItemEmpty, GroupItemEmpty.Text);
    end else
    begin
      Item := AddItem(NewMovie, GroupItemEmpty);
      UpdateGroupName(GroupItemEmpty, GetGroupNameWithoutCount(GroupItemEmpty.Text));
    end;
  end else
    Item := AddItem(NewMovie, GroupItemEmpty);

  ListView1.Selected := Item;
  ListView1.Selected.Focused := True;
  ListView1.Items.EndUpdate;
  ListView1AfterSelectionChange(ListView1);
  ListView1EnsureVisiblePlus;
  SetStatus;
  FCatalogFile.Modified := True;

end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ImportMoviePicture(const AFileName: TFileName; const ImportMethod: TPictureSelectOption);
var
  NewName: TFileName;
  ExistingName: TFileName;
  ExistingInSamePath: Boolean;
  Ext: string;
  CatalogPath: TFileName;
  Stream: TFileStream;
begin
    with ListView1 do
      if Selected <> nil then
        with Selected do
          if (Selected) and (Data <> nil) then
          begin

            if(MovieSave = nil) then
            begin
              MovieSave := TMovie.Create;
              MovieSave.Assign(ListView1.Selected.Data);
            end;

            with TMovie(Data) do
            begin
              try
                Stream := TFileStream.Create(AFileName, fmOpenRead);
                try
                  Ext := GetRealPictureExt(Stream, extImage[extPNG], extImage[extJPG], extImage[extGIF], ExtractFileExt(AFileName))
                finally
                  Stream.Free;
                end;
                if Ext = '' then
                  raise Exception.Create('Unrecognized picture format');
                CatalogPath := ExtractFilePath(FCatalogFile.CurrentFile);
                SetCurrentDir(CatalogPath);
                ExistingInSamePath := False;
                if (strPicture <> '') and (Picture = nil) then
                begin
                  ExistingName := ExpandFileName(strPicture);
                  if not FileExists(ExistingName) then
                    ExistingName := ''
                  else
                    ExistingInSamePath := Pos('\', strPicture) = 0;
                end
                else
                  ExistingName := '';
                if ImportMethod = psoStore then
                begin
                  if (ExistingName <> '') then
                  begin // there was a linked picture
                    if ExistingInSamePath then
                      DeleteFile(ExistingName)
                    else // it was not in same path than catalog, so not sure that it can be deleted
                      case MessageWin.Execute(Format(messages.Strings[msgPicReplaceLink], [ExistingName]), mtConfirmation, [mbYes, mbNo, mbCancel]) of
                        1:    DeleteFile(ExistingName);
                        0, 3: Exit;
                      end;
                  end;
                  try
                    strPicture := Ext;
                    if IndexText(strPicture, extImage) <> -1 then
                    begin
                      Picture.Free;
                      Picture := TMemoryStream.Create;
                      Picture.LoadFromFile(AFileName);
                    end;
                  except
                    on e: Exception do
                      begin
                        Picture.Free;
                        Picture := nil;
                        strPicture := '';
                        raise e;
                      end;
                  end;
                end
                else
                begin
                  if ImportMethod = psoCopy then
                  begin
                    if CatalogPath = '' then
                      raise Exception.Create(messages.Strings[msgPicCopyNoCatPath]);
                    if (not ExistingInSamePath) then
                    begin
                      case Settings.rOptions.rMovieInformation.rPicImport.Naming of
                        1:  NewName := IntToStr(iNumber) + Ext;
                        2:  NewName := ValidateFileName(GetFormattedTitle) + Ext;
                      else
                        NewName := ChangeFileExt(ExtractFileName(AFileName), Ext);
                      end;
                      if Settings.rOptions.rMovieInformation.rPicImport.CatalogPrefix then
                        NewName := Format('%s_%s', [ChangeFileExt(ExtractFileName(FCatalogFile.CurrentFile), ''), NewName]);
                      if (ExistingName <> '') then
                        case MessageWin.Execute(Format(messages.Strings[msgPicReplaceLink], [ExistingName]), mtConfirmation, [mbYes, mbNo, mbCancel]) of
                          1:    DeleteFile(ExistingName);
                          0, 3: Exit;
                        end;
                      NewName := ExtractFileName(MakeUniqueFileName(CatalogPath + NewName, '_'));
                    end
                    else
                      NewName := ExtractFileName(ExistingName);
                    if not CopyFile(PChar(AFileName), PChar(CatalogPath + NewName), False) then
                      RaiseLastOSError;
                    strPicture := NewName;
                    Picture.Free;
                    Picture := nil;
                  end
                  else
                  begin
                    if ExistingInSamePath then
                      DeleteFile(ExistingName)
                    else
                    if (ExistingName <> '') then
                      case MessageWin.Execute(Format(messages.Strings[msgPicReplaceLink], [ExistingName]), mtConfirmation, [mbYes, mbNo, mbCancel]) of
                        1:    DeleteFile(ExistingName);
                        0, 3: Exit;
                      end;
                    if ImportMethod = psoLinkRel then
                      NewName := ExtractRelativePath(CatalogPath, AFileName)
                    else
                      NewName := AFileName;
                    strPicture := NewName;
                    Picture.Free;
                    Picture := nil;
                  end;
                end;
                FCatalogFile.Modified := True;
                MovieSelected;
              except
                on e: Exception do
                  MessageWin.Execute(Format(Messages.Strings[msgPicLoadFailed], [e.Message]), mtError, [mbOk]);
              end;
              if(_thumb <> nil) then
                FreeAndNil(_thumb);
              _thumbError := 0;
            end;
            HTMLViewerUpdate;
          end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.LoadMoviePicture;
var
  loadedPic: TGraphic;
  PicSizeDouble: Double;
begin
  with ListView1 do
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data <> nil) then
        begin
          with MoviePicture do
          begin
            Hint := '|' + GetLongHint(Hint);
            Picture.Assign(nil);
            if Data = nil then
            begin
              Width := 0;
              Height := 0;
              Exit;
            end;
          end;
          if ToolbarPictureWindow.Visible then
          begin
            with TMovie(Data) do
            begin
              TMovie(Data).Lock;
              try
                if Picture = nil then
                begin
                  if strPicture = '' then
                    Abort
                  else
                  begin
                    if FCatalogFile.CurrentFile <> '' then
                      SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
                    MoviePicture.Picture.LoadFromFile(strPicture);
                  end;
                  PicSizeDouble := GetFileSize(strPicture);
                  MoviePicture.Hint := Format('%s%s%s%s', [
                    Format(Messages.Strings[msgPicHintLinked], [strPicture]),
                    sLineBreak,
                    Format(Messages.Strings[msgPicHintInfo], [PicSizeDouble]),
                    MoviePicture.Hint
                  ]);
                end else
                begin
                  case IndexText(strPicture, extImage) of
                    extPNG:
                      loadedPic := TPNGObject.Create;
                    extJPG, extJPE, extJPEG:
                      loadedPic := TJPEGImageFix.Create;
                    extGIF:
                      loadedPic := TJvGIFImage.Create;
                  else
                    loadedPic := nil;
                    Abort;
                  end;
                  PicSizeDouble := Picture.Size;
                  MoviePicture.Hint := Format('%s%s%s%s', [
                    Format(Messages.Strings[msgPicHintStored], [
                      typeImage[IndexText(strPicture, extImage)]
                    ]),
                    sLineBreak,
                    Format(Messages.Strings[msgPicHintInfo], [
                      PicSizeDouble
                    ]),
                    MoviePicture.Hint
                  ]);
                  Picture.Seek(0, soFromBeginning);
                  loadedPic.LoadFromStream(Picture);
                  MoviePicture.Picture.Assign(loadedPic);
                  loadedPic.Free;
                end;
              except
                with MoviePicture do
                begin
                  Width:=0;
                  Height:=0;
                end;
              end;
              TMovie(Data).UnLock;
            end;
          end;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.OpenItem(const AMovie: TMovie): integer;
begin
  //SetCurrentDir(strDirApp);
  FrmMovie.LoadFromObject(AMovie);
  HTMLViewerUpdate;
  LoadMoviePicture;
  result := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.OpenItem(const Item: TElTreeItem): integer;
begin
  Result := -1;
  if (Item <> nil) and (Item.Data <> nil) then
  begin
    OpenItem(TMovie(Item.Data));
    FPreviousItem := Item;
    Result := 0;
  end else
  begin
    FPreviousItem := nil;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.SaveCurrentItem(const AMovie: TMovie): Integer;
begin
  if (AMovie = nil) then
  begin
    Result := -1;
    Exit;
  end;
  with FrmMovie, AMovie, Settings.rOptions.rFiles do
    if History and (strBorrower <> EBorrower.Text) then
    begin
      SetCurrentDir(strDirApp);
      with TLoanHistory.Create(ExpandFileName(HistoryFile), ExtractFileName(FCatalogFile.CurrentFile)) do
      try
        if strBorrower <> '' then
          Add(strBorrower, iNumber, strMedia, GetFormattedTitle, lhIn);
        if EBorrower.Text <> '' then
          Add(EBorrower.Text, iNumber, strMedia, GetFormattedTitle, lhOut);
      finally
        Free;
      end;
    end;

  // Update current item in the list if needed
  if FrmMovie.Modified then
  begin
    //ShowMessage('Current list item need to be updated !');
    FrmMovie.SaveToObject(AMovie);
    UpdateItem(FPreviousItem);
  end;
  FreeAndNil(MovieSave);

  UpdateLists;
  Result := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.SaveCurrentItem: Integer;
begin
  Result := -1;
  if (FPreviousItem = nil) or (FPreviousItem.Data = nil) then
    Exit;
  //if ListView1.Enabled and self.Enabled then
  //  ListView1.SetFocus;
  Result := SaveCurrentItem(TMovie(FPreviousItem.Data));
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.AddItem(Movie: TMovie; GroupItem: TElTreeItem) : TElTreeItem;
var
  j: Integer;
begin
  Inc(MoviesVisible);
  Result := ListView1.Items.AddChild(GroupItem, Movie.GetFormattedTitle(ColumnTitleDisplay, UsePrefixes));
  Movie._listItems.Add(Result);
  // *** Adding each item to the list ***
  with Result do
  begin
    Result.SubItems.Add(IntToStr(Movie.iNumber));
    if ActionToolsGrid.Checked then
    begin
      SubItems.Add('');
      SubItems.Add(IntToStr(Movie.iNumber));
      for j := 0 to fieldCount-1 do
        if not (j in [fieldNumber, fieldFormattedTitle]) then
        begin
          if (Settings.rOptions.rMovieList.GridTextSize > 0) and
            not (j in [fieldFormattedTitle, fieldOriginalTitle, fieldTranslatedTitle]) then
          begin
            SubItems.Add(Copy(Movie.GetFieldValue(j, True), 1, Settings.rOptions.rMovieList.GridTextSize));
          end else
          begin
            SubItems.Add(Movie.GetFieldValue(j, True));
          end;
        end;
    end else
      SubItems.Add('');
    Checked := Movie.bChecked;
    Data := Movie;
    ShowCheckBox := True;
    Tag := 0;
  end; // with Item
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

(* procedure TMainWindow.ModifyItem(Item: TElTreeItem; MovieTmp: TMovie);
var
  Modifs: Boolean;
  FieldValue: string;
  i, j: Integer;
  List: TObjectList;
  SameItem: TElTreeItem;

  procedure UpdateI(Field: Integer; Text: string);
  var
    ListField: Integer;
  begin
    ListField := Field;
    if ListField >= fieldFormattedTitle then
      Dec(ListField);
    ListField := ListField + 2;
    if (Settings.rOptions.rMovieList.GridTextSize > 0) and (Field <> fieldFormattedTitle) and
       (Field <> fieldTranslatedTitle) and (Field <> fieldOriginalTitle) then
      FieldValue := Copy(Text, 1, Settings.rOptions.rMovieList.GridTextSize)
    else
      FieldValue := Text;
    if (AnsiCompareStr(FieldValue, Item.SubItems.Strings[ListField]) <> 0) then
    begin
      Item.SubItems.Strings[ListField] := FieldValue;
      if not Modifs then
      begin
        Modifs := True;
        ListView1.Items.BeginUpdate;
      end;
    end;
  end;

begin
  if (MovieTmp <> nil) and (Item <> nil) and (Item.Data <> nil) then
  begin
    Modifs := False;
    if ActionToolsGrid.Checked then
    begin
      for i := 0 to fieldCount-1 do
        UpdateI(i, MovieTmp.GetFieldValue(i));
    end else
    begin
      FieldValue := MovieTmp.GetFormattedTitle;
      if (AnsiCompareStr(FieldValue, Item.Text) <> 0) then
      begin
        Modifs := True;
        ListView1.Items.BeginUpdate;
        Item.Text := FieldValue;
      end;
      FieldValue := MovieTmp.GetFieldValue(fieldNumber);
      if (AnsiCompareStr(FieldValue, Item.SubItems.Strings[0]) <> 0) then
      begin
        if(Modifs = False) then
        begin
          Modifs := True;
          ListView1.Items.BeginUpdate;
        end;
        Item.SubItems.Strings[0] := FieldValue;
      end;
      if (Item.Checked <> MovieTmp.bChecked) then
      begin
        if(Modifs = False) then
        begin
          Modifs := True;
          ListView1.Items.BeginUpdate;
        end;
        Item.Checked := MovieTmp.bChecked;
      end;
    end;
    if Modifs then
    begin
      List := TObjectList(TMovie(Item.Data)._listItems);
      for i := 0 to List.Count-1 do
      begin
        SameItem := TElTreeItem(List.Items[i]);
        if SameItem <> Item then
        begin
          for j := 0 to SameItem.SubItems.Count-1 do
          begin
            SameItem.SubItems.Strings[j] := Item.SubItems.Strings[j];
          end;
          SameItem.Text := Item.Text;
        end;
      end;
      ListView1.Items.EndUpdate;
      ListView1EnsureVisiblePlus;
    end;
  end;
end; *)

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.DeleteItem(Movie: TMovie; const CanShowWindow, ShowAllButtons: Boolean): Integer;
var
  DoNotAsk: Boolean;
  Buttons: array of Variant;
  Res, i: Integer;
  Item, ItemParent: TElTreeItem;
begin
  if ShowAllButtons then
    Buttons := VarArrayOf([mbYes,mbYesToAll,mbNo,mbNoToAll])
  else
    Buttons := VarArrayOf([mbYes,mbNo]);
  Result := -1;
  if Movie <> nil then
  begin
    if Settings.rOptions.rMovieList.ConfirmDelete and CanShowWindow then
    begin
      DoNotAsk := false;
      Res := MessageWin.Execute(Format(Messages.Strings[msgDeleteMovie],
        [Movie.GetFieldValue(fieldNumber), Movie.GetFormattedTitle]),
        mtConfirmation, DoNotAsk, Messages.Strings[msgDoNotConfirm], Buttons);
      if (not ShowAllButtons) and (Res = 2) then
        Res := 3;
      case Res of
        1:
          begin
            Settings.rOptions.rMovieList.ConfirmDelete := not DoNotAsk;
            Result := 1;
          end;
        2:
          begin
            Settings.rOptions.rMovieList.ConfirmDelete := not DoNotAsk;
            if DoNotAsk then
              Result := 1
            else
              Result := 2;
          end;
        3:
          begin
            Result := 3;
            Exit;
          end;
        4,0:
          begin
            Result := 4;
            Exit;
          end;
      end;
    end;
    ListView1.Items.BeginUpdate;
    for i := 0 to Movie._listItems.Count-1 do
    begin
      Item := TElTreeItem(Movie._listItems.Items[i]);
      ItemParent := Item.Parent;
      if Item = ListView1.ItemFocused then
        ListView1.ItemFocused := nil;
      Listview1.Items.Delete(Item);
      Dec(MoviesVisible);
      if (ItemParent <> nil) then
      begin
        if ItemParent.Count = 0  then
        begin
          if ItemParent = GroupItemEmpty then
            GroupItemEmpty := nil;
          if ItemParent = ListView1.ItemFocused then
            ListView1.ItemFocused := nil;
          Listview1.Items.Delete(ItemParent)
        end else
          UpdateGroupName(ItemParent, GetGroupNameWithoutCount(ItemParent.Text));
      end;
    end;
    ListView1.Items.EndUpdate;
    MovieList.Remove(Movie);
    if Result = -1 then
      Result := 0;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

// Update tree item with informations in FrmMovie !
procedure TMainWindow.UpdateItem(Item: TElTreeItem);
var
  Modifs: Boolean;
  FieldValue: string;
  i, j: Integer;
  List: TObjectList;
  SameItem: TElTreeItem;

  procedure UpdateI(Field: Integer; Text: string);
  var
    ListField: Integer;
  begin
    ListField := Field;
    if ListField >= fieldFormattedTitle then
      Dec(ListField);
    ListField := ListField + 2;
    if (Settings.rOptions.rMovieList.GridTextSize > 0) and (Field <> fieldFormattedTitle) and
       (Field <> fieldTranslatedTitle) and (Field <> fieldOriginalTitle) then
      FieldValue := Copy(Text, 1, Settings.rOptions.rMovieList.GridTextSize)
    else
      FieldValue := Text;
    if (AnsiCompareStr(FieldValue, Item.SubItems.Strings[ListField]) <> 0) then
    begin
      Item.SubItems.Strings[ListField] := FieldValue;
      if not Modifs then
      begin
        Modifs := True;
        ListView1.Items.BeginUpdate;
      end;
    end;
  end;

begin
  if (Item <> nil) and (Item.Data <> nil) then
  begin
    Modifs := False;
    if ActionToolsGrid.Checked then
    begin
      with FrmMovie do
      begin
          UpdateI(fieldMedia, EMedia.Text);
          UpdateI(fieldMediaType, FrmMovie.EMediaType.Text);
          UpdateI(fieldSource, FrmMovie.ESource.Text);
          if FrmMovie.EDate.Checked then
            UpdateI(fieldDate, DateToStr(Trunc(FrmMovie.EDate.DateTime)))
          else
            UpdateI(fieldDate, '');
          UpdateI(fieldBorrower, FrmMovie.EBorrower.Text);
          UpdateI(fieldRating, FrmMovie.ERating.Text);
          UpdateI(fieldOriginalTitle, FrmMovie.EOriginalTitle.Text);
          UpdateI(fieldTranslatedTitle, FrmMovie.ETranslatedTitle.Text);
          UpdateI(fieldDirector, FrmMovie.EDirector.Text);
          UpdateI(fieldProducer, FrmMovie.EProducer.Text);
          UpdateI(fieldCountry, FrmMovie.ECountry.Text);
          UpdateI(fieldCategory, FrmMovie.ECategory.Text);
          UpdateI(fieldYear, FrmMovie.EYear.Text);
          UpdateI(fieldLength, FrmMovie.ELength.Text);
          UpdateI(fieldActors, FrmMovie.EActors.Text);
          UpdateI(fieldURL, FrmMovie.EURL.Text);
          UpdateI(fieldDescription, FrmMovie.EDescription.Text);
          UpdateI(fieldComments, FrmMovie.EComments.Text);
          UpdateI(fieldVideoFormat, FrmMovie.EVideoFormat.Text);
          UpdateI(fieldVideoBitrate, FrmMovie.EVideoBitrate.Text);
          UpdateI(fieldAudioFormat, FrmMovie.EAudioFormat.Text);
          UpdateI(fieldAudioBitrate, FrmMovie.EAudioBitrate.Text);
          UpdateI(fieldResolution, FrmMovie.EResolution.Text);
          UpdateI(fieldFrameRate, FrmMovie.EFramerate.Text);
          UpdateI(fieldLanguages, FrmMovie.ELanguages.Text);
          UpdateI(fieldSubtitles, FrmMovie.ESubtitles.Text);
          UpdateI(fieldSize, FrmMovie.ESize.Text);
          UpdateI(fieldDisks, FrmMovie.EDisks.Text);
          UpdateI(fieldNumber, TMovie(Item.Data).GetFieldValue(fieldNumber));
          if Modifs then
            Item.Text := GetTitle;
        //Item.SubItems.Strings[2] := TMovie(Item.Data).GetFieldValue(fieldNumber);
        //Item.Checked := TMovie(Item.Data).bChecked;
      end;
    end else
    begin
      FieldValue := GetTitle;
      if (AnsiCompareStr(FieldValue, Item.Text) <> 0) then
      begin
        Modifs := True;
        ListView1.Items.BeginUpdate;
        Item.Text := FieldValue;
        //Item.SubItems.Strings[0] := TMovie(Item.Data).GetFieldValue(fieldNumber);
        //Item.Checked := TMovie(Item.Data).bChecked;
      end;
    end;
    if Modifs then
    begin
      //FrmMovie.SaveToObject(TMovie(Item.Data));
      List := TObjectList(TMovie(Item.Data)._listItems);
      for i := 0 to List.Count-1 do
      begin
        SameItem := TElTreeItem(List.Items[i]);
        if SameItem <> Item then
        begin
          for j := 0 to SameItem.SubItems.Count-1 do
          begin
            SameItem.SubItems.Strings[j] := Item.SubItems.Strings[j];
          end;
          SameItem.Text := Item.Text;
        end;
      end;
      ListView1.Items.EndUpdate;
      ListView1EnsureVisiblePlus;
      ThumbsViewerStart;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.AddGroupItem(GroupName: string) : TElTreeItem;
begin
  Result := ListView1.Items.AddChild(nil, GroupName);
  with Result do
  begin
    Data := nil;
    Tag := 0;
    Expanded := Settings.rOptions.rMovieList.GroupExpand;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.GetGroupNameWithoutCount(GroupName: string) : string;
var
  lastPosGroupName: Integer;
begin
  if Settings.rOptions.rMovieList.GroupCount then
  begin
     lastPosGroupName := LastPos(' (', GroupName) - 1;
     Result := Copy(GroupName, 1, lastPosGroupName)
  end else
    Result := GroupName;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.UpdateGroupName(GroupItem: TElTreeItem; GroupNameWithoutCount: string);
begin
  if Settings.rOptions.rMovieList.GroupCount and (GroupItem <> nil) and (GroupItem.Data = nil) then
    GroupItem.Text := Format('%s (%d)', [GroupNameWithoutCount, GroupItem.Count])
  else
    GroupItem.Text := GroupNameWithoutCount;
end;

{-------------------------------------------------------------------------------
  File Management
-------------------------------------------------------------------------------}

procedure TMainWindow.OnFileChange(Sender: TObject; AFileName: TFileName; var Result: Boolean);
begin
  Result := True;
  SetTitle;
  SetStatus;
  MovieSelected;
  SaveLists;
  LoadLists;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.OnFileModified(Sender: TObject; State: Boolean);
begin
  with StatusBar1.Panels.Items[panelModified] do
    if State then
      Caption := Messages.Strings[msgUnsavedStatus]
    else
      Caption := '';
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.OnNewFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
begin
  ThumbsViewerStop;
  Result := True;
  FPreviousItem := nil;
  FCatalogFile.Modified := False;
  ClearItem;
  MovieList.Free;
  MovieList := TMovieList.Create;
  RefreshMovieList(False); //ThumbsViewerStart;
  UpdateMRU(AFileName);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.OnOpenFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
begin
  ThumbsViewerStop;
  Result := True;
  ClearItem;
  ListView1.DeselectAll;
  MovieSelected;
  try
    Application.ProcessMessages;
    with ProgressWin do
    begin
      Maximum := 2;
      Status := Format(Messages.Strings[msgLoading],[ExtractFileName(AFileName)]);
      IntProgress := 0;
      Execute(Self);
      try
        FPreviousItem := nil;
        MovieList.Free;
        MovieList := TMovieList.Create;
        if LowerCase(ExtractFileExt(AFileName)) = extCatalog[extXML] then
          MovieList.LoadFromXML(AFileName)
        else
          MovieList.LoadFromFile(AFileName);
        IntProgress := Maximum-1;
        RefreshMovieList(False); //ThumbsViewerStart;
        FCatalogFile.Modified := False;
        UpdateMRU(AFileName);
        IntProgress := Maximum;
      finally
        Close;
      end;
    end;
  except
    on E: Exception do
    begin
      ProgressWin.Close;
      Application.ProcessMessages;
      FCatalogFile.New;
      MessageWin.Execute(E.Message,mtError,[mbOk]);
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.OnSaveFile(Sender: TObject; AFileName: TFileName; var Result: Boolean);
var
  Res, SaveFormat: Integer;
begin
  ThumbsViewerStop;
  Result := True;
  SaveCurrentItem; // To be sure that the current item is also saved
  with ProgressWin do
  begin
    Application.ProcessMessages;
    Maximum := 2;
    Status := Format(Messages.Strings[msgSaving],[ExtractFileName(AFileName)]);
    IntProgress := 0;
    Execute(Self);
    try
      try
        MovieList.Sort(fieldNumber);
        DeleteFile(ChangeFileExt(AFileName,'.bak'));
        RenameFile(AFileName, ChangeFileExt(AFileName,'.bak'));
        SaveFormat := -1;
        case IndexText(ExtractFileExt(AFileName), extCatalog) of
          extAMC:
            SaveFormat := extAMC;
          extXML:
            SaveFormat := extXML;
        else
          with FCatalogFile.SaveDialog do
            case FilterIndex of
              DialogCatalogFilterAMC:
                SaveFormat := extAMC;
              DialogCatalogFilterXML:
                SaveFormat := extXML;
            end;
        end;
        case SaveFormat of
          extAMC:
            MovieList.SaveToFile(AFileName);
          extXML:
            begin
              if MovieList.HasImages(True, False, False) then
                Res := MessageWin.Execute(InsertLineBreaks(Messages.Strings[msgXmlConvertImages]), mtConfirmation, [mbOk, mbIgnore, mbCancel])
              else
                Res := 2;
              if not (Res in [0, 3]) then
                MovieList.SaveToXML(AFileName, Res = 1{, Settings.rOptions.rFiles.XMLHeader})
              else
              begin
                Result := False;
                Exit;
              end;
            end;
        end;
        UpdateMRU(AFileName);
        if not Settings.rOptions.rFiles.Backup then
        begin
          DeleteFile(ChangeFileExt(AFileName,'.bak'));
        end;
        IntProgress := Maximum;
      finally
        Close;
      end;
      Application.ProcessMessages;
    except
      on E: Exception do
      begin
        MessageWin.Execute(E.Message,mtError,[mbOk]);
        Result := False;
      end;
    end;
  end;
  ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.OnDialogTypeChange(Sender: TObject);
var
  NewName, NewExt: string;
begin
  if Sender is TSaveDialog then
    with Sender as TSaveDialog do
    begin
      if not DirectoryExists(FileName) then
      begin
        case FilterIndex of
          DialogCatalogFilterAMC:
            begin
              NewName := ChangeFileExt(ExtractFileName(FileName), extCatalog[extAMC]);
              NewExt := extCatalog[extAMC];
            end;
          DialogCatalogFilterXML:
            begin
              NewName := ChangeFileExt(ExtractFileName(FileName), extCatalog[extXML]);
              NewExt := extCatalog[extXML];
            end;
        end;
        Delete(NewExt, 1, 1);
        SendMessage(Windows.GetParent(Handle), CDM_SETCONTROLTEXT, 1152, Integer(PChar(NewName)));
        SendMessage(Windows.GetParent(Handle), CDM_SETDEFEXT, 1152, Integer(PChar(NewExt)));
      end;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetTitle;
var
  FileName: string;
begin
  if FCatalogFile.CurrentFile = '' then
    FileName := '<untitled>'
  else
    FileName := FCatalogFile.CurrentFile;
  Caption := Format(strDefaultCaption,[strVersion, FileName]);
  {$IFDEF DISABLETHEMES}
  Caption := Format('%s [themes support disabled]', [Caption]);
  {$ENDIF}
  {$IFDEF ANTDEBUG}
  Caption := Format('%s [BETA/DEBUG]', [Caption]);
  {$ENDIF}
  Application.Title := Format(strDefaultTask, [ExtractFileName(FileName)]);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetStatus;
var
  MoviesCount: Integer;
begin
  if MovieList <> nil then
    MoviesCount := MovieList.Count
  else
    MoviesCount := 0;
  with StatusBar1.Panels.Items[panelCount] do
  begin
    Caption := Format(Messages.Strings[msgStatusCount], [MoviesCount]) + ' ' +
      Format(Messages.Strings[msgStatusVisible], [MoviesVisible]);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.UpdateMRU(const strFilePath: string);
begin
  if strFilePath <> '' then
    TBMRUList1.Add(strFilePath);
  ActionFileOpenNoRecent.Visible := TBMRUList1.Items.Count = 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.TBMRUList1Click(Sender: TObject;
  const Filename: String);
begin
  FCatalogFile.Open(FileName)
end;

{-------------------------------------------------------------------------------
   Misc.
-------------------------------------------------------------------------------}

procedure TMainWindow.RefreshMovieList(const KeepSelection: Boolean = True);
var
  i, k: Integer;
  GroupItem, GroupItemNotAlone, Item: TElTreeItem;
  MovieFilter: TMovieFilter;
  GridMode, ItemInList: Boolean;
  NotAloneInGroup, GroupUnique, GroupEmpty, GroupCount, GroupExpand: Boolean;

  Groups : TStringList;
  GroupMovies: TObjectList;
  GroupName: string;
  iGroup, iMovie: Integer;
  Movie: TMovie;
  // Args: IN [GroupName] OUT [GroupItem]
  procedure AddGroupI;
  begin
    GroupItem := ListView1.Items.AddChild(nil, GroupName);
    with GroupItem do
    begin
      Data := nil;
      Tag := 0;
      Expanded := GroupExpand;
    end;
  end;

  // Args: IN [GroupItem]
  procedure AddGroupCount;
  begin
    if GroupCount and (GroupItem <> nil) and (GroupItem <> GroupItemNotAlone) then
      GroupItem.Text := Format('%s (%d)', [GroupItem.Text, GroupItem.Count]);
  end;

begin
  ThumbsViewerStop;
  if KeepSelection then
    StoreSelectedState;
  ListView1.DeselectAll;
  MovieSelected;
  try
    with Settings.rOptions do
    begin
      NotAloneInGroup := rMovieList.GroupNotAlone and rMovieList.GroupUnique; //rMovieList.GroupNotAlone;
      GroupUnique := rMovieList.GroupUnique;
      GroupEmpty := True;//rMovieList.GroupEmpty;
      GroupCount := rMovieList.GroupCount;
      GroupExpand := rMovieList.GroupExpand;
    end;
    GroupItem := nil;
    GroupItemNotAlone := nil;
    SaveCurrentItem;
    FPreviousItem := nil;
    MovieFilter := nil;
    MoviesVisible := 0;
    GroupItemEmpty := nil;
    if ActionFindDisplay.Checked then
    begin
      MovieFilter := TMovieFilter.Create;
      MovieFilter.value := EValue.Text;
      MovieFilter.field := cbxField.ItemIndex - 1;
      MovieFilter.wholeField := ActionFindWholefield.Checked;
      MoviesVisible := 0;
    end;
    GridMode := ActionToolsGrid.Checked;

    with ListView1 do
    begin
      Items.BeginUpdate;
      try
        Items.Clear;

        // *** Building headers ***
        with HeaderSections do
        begin
          if Count > 3 then
            Settings.rMain.ColumnSettings := SectionsOrder;
          i := Count-1;
          if i > 2 then
            while i >= 0 do
            begin
              if(Item[i].Visible) then
                DeleteSection(Item[i]);
              Dec(i);
            end
          else if i > 1 then
            DeleteSection(Item[2]);

          if GridMode then
          begin
            ListView1.OnCompareItems := ListView1CompareItems;
            Item[0].Visible := False;
            Item[1].Visible := False;
            with AddSection do
            begin
              Text := strFields.Strings[fieldNumber];
              FieldName := IntToStr(fieldNumber);
              FieldType := sftCustom;//sftNumber;
              Alignment := hsaRight;
            end;
            with AddSection do
            begin
              Text := strFields.Strings[fieldFormattedTitle];
              FieldName := IntToStr(fieldFormattedTitle);
              FieldType := sftCustom;//sftText;
            end;
            for i := 0 to fieldCount-1 do
            begin
              if not (i in [fieldNumber, fieldFormattedTitle]) then
              begin
                with AddSection do
                begin
                  Text := strFields.Strings[i];
                  FieldName := IntToStr(i);
                  case GetFieldType(i) of
                    ftInteger:
                      begin
                        FieldType := sftCustom;//sftNumber;
                        Alignment := hsaRight;
                      end;
                    ftReal:
                      begin
                        FieldType := sftCustom;//sftFloating;
                        Alignment := hsaRight;
                      end;
                    ftDate:
                      begin
                        FieldType := sftCustom;//sftDate;
                      end;
                  else
                    FieldType := sftCustom;//sftText;
                  end;
                end;
              end;
            end;
            MainTreeColumn := 3;
            HideHorzScrollBar := False;
            MoveColumnOnDrag := True;
            DraggableSections := True;
            // Set column order
            SectionsOrder := Settings.rMain.ColumnSettings;
          end else
          begin
            ListView1.OnCompareItems := ListView1CompareItems;
            with HeaderSections do
            with AddSection do
            begin
              Text := 'Invisible';
              FieldType := sftCustom;
              Visible := False;
              //showmessage(IntToStr(Index));
            end;
            Item[0].Visible := Settings.rOptions.rMovieList.MovieNumColumn;
            Item[0].FieldName  := IntToStr(fieldNumber);
            Item[1].Visible := True;
            Item[1].FieldName  := IntToStr(fieldFormattedTitle);
            MainTreeColumn := 1;
            HideHorzScrollBar := True;
            MoveColumnOnDrag := False;
            DraggableSections := False;
          end; // if gridmode
          SetSortField(Abs(FSortField) - 1, FSortField < 0);
        end; // with headers

        // *** Filling the list ***
        if (MovieList <> nil) then
        begin
          // *** Remove all associated items for a movie ***
          for k := 0 to MovieList.Count-1 do
            MovieList.Items[k]._listItems.Clear;

          // *** Group movies by selected field ***
          Groups := MovieList.GetMoviesByGroups(FGroupField, mioAll, MovieFilter);

          // *** For all groups ***
          for iGroup := 0 to Groups.Count-1 do
          begin
            AddGroupCount;
            GroupName := Groups.Strings[iGroup];
            GroupMovies := TObjectList(Groups.Objects[iGroup]);

            // ** If group wanted ***
            if FGroupField <> -1 then
              if NotAloneInGroup and (GroupMovies.Count = 1) and
                (AnsiCompareStr(GroupName, Messages.Strings[msgGroupEmpty]) <> 0) and
                (AnsiCompareStr(GroupName, strErrorParenthesis) <> 0) then
                if GroupUnique then
                  if GroupItemNotAlone = nil then
                  begin
                    GroupName := Messages.Strings[msgGroupUnique];
                    AddGroupI;
                    GroupItemNotAlone := GroupItem;
                  end else
                    GroupItem := GroupItemNotAlone
                else
                  GroupItem := nil
              else if GroupName = '$$$EMPTY$$$' then
                if (GroupEmpty) then
                begin
                  GroupName := Messages.Strings[msgGroupEmpty];
                  AddGroupI;
                  GroupItemEmpty := GroupItem; // We keep group empty for next add !
                end else
                   GroupItem := nil
              else
                AddGroupI;

            // *** For all movies in group ***
            for iMovie := 0 to GroupMovies.Count-1 do
            begin
              Movie := TMovie(GroupMovies.Items[iMovie]);
              with Movie do
              begin
                // *** Check if movie is already in group ***
                ItemInList := False;
                for k := 0 to Movie._listItems.Count-1 do
                begin
                  Item := TElTreeItem(Movie._listItems.Items[k]);
                  if Item.Parent = GroupItem then
                  begin
                    ItemInList := True;
                    break;
                  end;
                end;
                // *** If movie is not already in group ***
                if not ItemInList then
                begin
                  AddItem(Movie, GroupItem);
                end; // Check if movie is already in group !
              end; // with Movie
            end; // if movie is not already in group
          end; // for all groups
          AddGroupCount; // add last group count
          GroupItem := GroupItemNotAlone;
          GroupItemNotAlone := nil;
          AddGroupCount; // add group count for not alone group
          Groups.Free;
        end; // if movielist <> nil
      finally
      end;
    end;
    SetStatus;
  finally
    ListView1Resize(Self);
  end;
  if KeepSelection then
    LoadSelectedState;
  ListView1.Items.EndUpdate;
  ListView1EnsureVisiblePlus;
  ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.GetTitle: string;
begin
  with FrmMovie do
    Result := GetFormattedTitle(EOriginalTitle.Text, ETranslatedTitle.Text, EMedia.Text, ColumnTitleDisplay, UsePrefixes)
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.MovieSelected;
var
  i: Integer;
  bPic: Boolean;
  SelCount: Integer;
begin
  SelCount := ListView1.SelectedCount;
  ActionMovieSelGroup.Enabled := False;
  with ListView1 do
  begin
    //WARNING: Some bugs in TElTree with CTRL+Selection !
    //Need to check SelCount and Selected because when SelCount = 1, Selected can be nil !!!
    //ShowMessage('Je passe : '+'SelCount ='+IntToStr(SelCount)+' Selected := '+BoolToStr((Selected <> nil), True));
    if (SelCount = 1) then
      if (Selected <> nil) then
      begin
        if (Selected.Data = nil) then
        begin
          ActionMovieSelGroup.Enabled := (ActionDisplayThumbnails.Checked = False);
          SelCount := 0;
        end;
      end else
        SelCount := 0;
  end;
  with FrmMovie do
  begin
    for i := 0 to ControlCount-1 do
      Controls[i].Enabled := SelCount = 1;
    Refresh;
  end;
  if SelCount <> 1 then
    MoviePicture.Picture.Assign(nil);
  ActionMovieSelCheck.Enabled := (ActionDisplayThumbnails.Checked = False);
  ActionMovieSelUncheck.Enabled := (ActionDisplayThumbnails.Checked = False);
  ActionMovieDelete.Enabled := (SelCount > 0) and
    ((ActionDisplayThumbnails.Checked = False) or (ThumbsViewer.Selection.Count > 0));
  ActionMovieCheck.Enabled := (SelCount > 0) and
    ((ActionDisplayThumbnails.Checked = False) or (ThumbsViewer.Selection.Count > 0));
  ActionMovieUncheck.Enabled := (SelCount > 0) and
    ((ActionDisplayThumbnails.Checked = False) or (ThumbsViewer.Selection.Count > 0));
  ActionMovieSearch.Enabled := SelCount = 1;
  ActionMovieNumber.Enabled := SelCount = 1;
  ActionMovieUndo.Enabled := SelCount = 1;
  ActionMovieCopy.Enabled := SelCount = 1;
  ActionMoviePaste.Enabled := SelCount = 1;
  ActionPicSelect.Enabled := SelCount = 1;
  ActionMovieImportFiles.Enabled := (SelCount < 2);
  if SelCount = 1 then
    with ListView1.Selected do
    begin
      bPic := TMovie(Data).strPicture <> '';
      ActionPicDelete.Enabled := bPic;
      ActionPicCopy.Enabled := bPic;
      ActionPicSaveAs.Enabled := bPic;
    end
  else
  begin
    ActionPicSaveAs.Enabled := False;
    ActionPicDelete.Enabled := False;
    ActionPicCopy.Enabled := False;
  end;
  HTMLViewerUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.LoadLists;
  procedure LoadList(const ACombo: TComboBox; ddlIndex: Integer);
  begin
    with Settings.rOptions.rMovieInformation.rCombo[ddlIndex] do
    begin
      if UseCatalogValues then
      begin
        if MovieList <> nil then
          MovieList.GetValues(ACombo.Items, strFieldsDdl[ddlIndex])
        else
          ACombo.Items.Clear;
      end else
        ACombo.Items.Assign(Contents);
      ACombo.Sorted := Sort;
      ACombo.AutoComplete := AutoComplete;
    end;
  end;
begin
  with FrmMovie do
  begin
    LoadList(ECountry, ddlCountry);
    LoadList(ECategory, ddlCategory);
    LoadList(EVideoFormat, ddlVideo);
    LoadList(EAudioFormat, ddlAudio);
    LoadList(ELanguages, ddlLanguages);
    LoadList(ESubtitles, ddlSubtitles);
    LoadList(EBorrower, ddlBorrowers);
    LoadList(EFramerate, ddlFramerate);
    LoadList(EMediaType, ddlMediaType);
    LoadList(ESource, ddlSources);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.LoadOptions;
var
  ButtonName: string;
  TheButton: TTBCustomItem;
  i: Integer;
begin
  ToolbarFind.Floating := True;
  TBCustomLoadPositions(Self, XmlReadInt, XmlReadString, Settings.rMain.Toolbars);
  try
    with Settings.rMain.Toolbars.Items.ItemNamed[ToolbarMain.Name].Items do
      for i := 0 to Count-1 do
      begin
        ButtonName := Item[i].Properties.Value('Name');
        if ButtonName <> '' then
        begin
          TheButton := TTBCustomItem(Self.FindComponent(ButtonName));
          if TheButton <> nil then
            TheButton.Visible := Item[i].Properties.BoolValue('Visible', TheButton.Visible);
        end;
      end;
  except
  end;
  ActionDisplayMainToolbar.Checked := ToolbarMain.Visible;
  ActionDisplayPictureToolbar.Checked := ToolbarPicture.Visible;
  with Settings do
  begin
    case rMain.WindowState of
      0:
        begin
          WindowState := wsMinimized;
        end;
      1:
        begin
          WindowState := wsNormal;
          Width := rMain.WindowWidth;
          Height := rMain.WindowHeight;
          Left := rMain.WindowLeft;
          Top := rMain.WindowTop;
        end;
      2:
        begin
          WindowState := wsMaximized;
        end;
      else
        WindowState := wsNormal;
    end;
    with rMain do
    begin
      ActionDisplayStatusBar.Checked := Statusbar;
      ActionDisplayStatusBarExecute(Self);
      PanelLeft.Width := ListWidth;
      ToolbarPictureWindow.Height := PictureDockedHeight;
      TBMRUList1.Items.Assign(MRU);
      UpdateMRU('');
      FGroupField := Group;
      FSortField := SortField;
      ActionFindWholefield.Checked := FindWholefield;
      if GridMode then
        ActionToolsGrid.Execute;
      if ListAlClient then
        ActionStretchList.Execute;
      if DisplayHTML then
        ActionDisplayHTML.Execute;
      if DisplayThumbnails then
        ActionDisplayThumbnails.Execute;
    end;
  end; // with Settings do
  ActionMovieFind.Checked := ToolbarFind.Visible;
  ActionMoviePictureShow.Checked := ToolbarPictureWindow.Visible;
  SplitterBottomList.Visible := ToolbarPictureWindow.Visible and not ToolbarPictureWindow.Floating;
  ApplyOptions;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SaveLists;
  procedure SaveList(const ACombo: TComboBox; ddlIndex: Integer);
  begin
    with Settings.rOptions.rMovieInformation do
      if (not rCombo[ddlIndex].UseCatalogValues) and (rCombo[ddlIndex].AutoAdd) and (ACombo.Items.Count > 0) then
        rCombo[ddlIndex].Contents.Assign(ACombo.Items);
  end;
begin
  with FrmMovie do
  begin
    SaveList(ECountry, ddlCountry);
    SaveList(ECategory, ddlCategory);
    SaveList(EVideoFormat, ddlVideo);
    SaveList(EAudioFormat, ddlAudio);
    SaveList(ELanguages, ddlLanguages);
    SaveList(ESubtitles, ddlSubtitles);
    SaveList(EBorrower, ddlBorrowers);
    SaveList(EFramerate, ddlFramerate);
    SaveList(EMediaType, ddlMediaType);
    SaveList(ESource, ddlSources);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.UpdateLists;
  procedure UpdateList(const Combo: TComboBox; DdlIndex: Integer);
  begin
    with Settings.rOptions.rMovieInformation.rCombo[DdlIndex] do
      if (AutoAdd or UseCatalogValues) and (Combo.Text <> '') and (Combo.Items.IndexOf(Combo.Text) = -1) then
        Combo.Items.Add(Combo.Text);
  end;
begin
  with FrmMovie do
  begin
    UpdateList(ECountry, ddlCountry);
    UpdateList(ECategory, ddlCategory);
    UpdateList(EVideoFormat, ddlVideo);
    UpdateList(EAudioFormat, ddlAudio);
    UpdateList(ELanguages, ddlLanguages);
    UpdateList(ESubtitles, ddlSubtitles);
    UpdateList(EBorrower, ddlBorrowers);
    UpdateList(EFramerate, ddlFramerate);
    UpdateList(EMediaType, ddlMediaType);
    UpdateList(ESource, ddlSources);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SaveOptions;
var
  i: Integer;
begin
  with Settings do
  begin
    case WindowState of
      wsMinimized:
        rMain.WindowState := 0;
      wsNormal:
        begin
          rMain.WindowState := 1;
          rMain.WindowWidth := Width;
          rMain.WindowHeight := Height;
          rMain.WindowLeft := Left;
          rMain.WindowTop := Top;
        end;
      wsMaximized:
        rMain.WindowState := 2;
    end;
    with rMain do
    begin
      Statusbar := ActionDisplayStatusBar.Checked;
      if ActionStretchList.Checked then
        ListWidth := PanelLeftOldWidth
      else
        ListWidth := PanelLeft.Width;
      PictureDockedHeight := ToolbarPictureWindow.Height;
      MRU.Assign(TBMRUList1.Items);
      Group := FGroupField;
      SortField := FSortField;
      GridMode := ActionToolsGrid.Checked;
      ListAlClient := ActionStretchList.Checked;
      DisplayHTML := ActionDisplayHTML.Checked;
      DisplayThumbnails := ActionDisplayThumbnails.Checked;
      FindWholefield := ActionFindWholefield.Checked;
      FindField := cbxField.ItemIndex - 1;
      with ListView1 do
      begin
        with HeaderSections do
          if Count > 3 then
            ColumnSettings := SectionsOrder;
      end;
    end;
    version := strVersion;

    // Save Toolbars
    Settings.rMain.Toolbars.Clear;
    TBCustomSavePositions(Self, XmlWriteInt, XmlWriteString, Settings.rMain.Toolbars);
    with Settings.rMain.Toolbars.Items.ItemNamed[ToolbarMain.Name].Items do
      for i := 0 to ToolbarMain.Items.Count-1 do
        with Add('Button') do
        begin
          Properties.Add('Name', ToolbarMain.Items.Items[i].Name);
          Properties.Add('Visible', ToolbarMain.Items.Items[i].Visible);
        end;

    // Save drop-down lists
    SaveLists;

    // Write to file
    Save;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ApplyOptions;
var
  bState:  Boolean;
  tb_color, tb_grey: TBitmap;
  ToolbarColorType: Integer;
  s: string;
  ClassicIconTheme: Boolean;
  procedure SetToolbarImages(const AImageList: TCustomImageList);
  begin
    if Settings.rOptions.rDisplay.ImagesInMenu then
    begin
      ToolbarMenu.Images := AImageList;
      PopupMovieList.Images := AImageList;
      PopupToolbar.Images := AImageList;
    end;
    ToolbarMain.Images := AImageList;
    ToolbarPicture.Images := AImageList;
    ToolbarImages := AImageList;
  end;
begin
  with Settings.rOptions do
  begin

    { --- Toolbar icons, skin & colors --- }

    tb_color := TBitmap.Create;
    tb_grey := TBitmap.Create;
    try
      ClassicIconTheme := True;
      s := strDirApp + strDirToolbars + rDisplay.IconSet + '.bmp';
      if FileExists(s) then
        try
          tb_color.LoadFromFile(s);
          ClassicIconTheme := False;
        except
        end;
      if ClassicIconTheme then
        tb_color.LoadFromResourceName(HInstance, 'TOOLBARCLASSIC');
      tb_color.PixelFormat := pf24bit;
      ToolbarColorType := rDisplay.ColorType;
      if ToolbarColorType = 0 then
        SetToolbarImages(ImageListHot)
      else
      begin
        SetToolbarImages(ImageListNormal);
        tb_grey.Assign(tb_color);
        case ToolbarColorType of
          1: GrayScale(tb_grey);
          2: Blend(tb_grey, clWhite);
          3: Blend(tb_grey, clDkGray);
        end;
        ImageListNormal.Clear;
        ImageListNormal.Height := tb_grey.Height;
        ImageListNormal.Width := ImageListNormal.Height;
        ImageListNormal.AddMasked(tb_grey, tb_grey.Canvas.Pixels[0, 0]);
      end;
      ImageListHot.Clear;
      ImageListHot.Height := tb_color.Height;
      ImageListHot.Width := ImageListHot.Height;
      ImageListHot.AddMasked(tb_color, tb_color.Canvas.Pixels[0, 0]);
    finally
      tb_color.Free;
      tb_grey.Free
    end;
    with ImageListHot do
    begin
      GetIcon(Ord(ICON_MOVIENUMBER), NumberWin.Icon);
    end;
    {$IFNDEF DISABLETHEMES}
    if rDisplay.OfficeXP then
      TBXSwitcher1.Theme := 'OfficeXP'
    else
      TBXSwitcher1.Theme := 'Default';
    {$ENDIF}


    { --- Comboboxes --- }

    SaveLists;
    LoadLists;

    { -- Other options --- }

    TBMRUList1.MaxItems := rFiles.RecentFiles;

    ColumnTitleDisplay := rMovieList.TitleColumn;
    UsePrefixes := rMovieList.UsePrefixes;

    bState := rMovieList.Checkboxes;
    listview1.ShowCheckboxes := bState;
    ActionMovieCheck.Enabled := bState;
    ActionMovieUncheck.Enabled := bState;
    ActionMovieSelCheck.Enabled := bState;
    ActionMovieSelUncheck.Enabled := bState;

    with ListView1 do
    begin
      Tracking := rMovieList.HotTrack;
      UseCustomScrollBars := rMovieList.EnhancedScrollbars;
    end;

    ActionMoviePrevious.ShortCut := rMovieList.ShortcutPrev;
    ActionMovieNext.ShortCut := rMovieList.ShortcutNext;

    s := FrmMovie.ERating.Hint;
    if rMovieInformation.RatingTrunc then
      FrmMovie.ERating.Decimal := 0
    else
      FrmMovie.ERating.Decimal := 1;

    if rMovieInformation.PictureBackground = clDefault then
      ScrollBox1.Color := clBtnFace
    else
      ScrollBox1.Color := rMovieInformation.PictureBackground;

    if rMovieInformation.PictureFitWindow then
    begin
      MoviePicture.Align := alClient;
      MoviePicture.Stretch := true;
    end else
    begin
      MoviePicture.Align := alNone;
      MoviePicture.Stretch := false;
    end;

    if rDisplay.SoftBorders then
    begin
      with ListView1 do
      begin
        Flat := True;
        HeaderFlat := True;
        VertScrollBarStyles.ActiveFlat := True;
        VertScrollBarStyles.Flat := True;
      end;
      ScrollBox1.BorderStyle := bsNone;
      ScrollBox1.BevelKind := bkTile;
      PanelMovieInfos.BorderStyle := bsNone;
      PanelMovieInfos.BevelOuter := bvLowered;
    end else
    begin
      with ListView1 do
      begin
        Flat := False;
        HeaderFlat := IsThemedXP;
        VertScrollBarStyles.ActiveFlat := False;
        VertScrollBarStyles.Flat := False;
      end;
      ScrollBox1.BorderStyle := bsSingle;
      ScrollBox1.BevelKind := bkNone;
      PanelMovieInfos.BorderStyle := bsSingle;
      PanelMovieInfos.BevelOuter := bvNone;
    end;

    RebuildSearchMenu;

  end; // with Settings.rOptions

  FrmMovie.LSizeUnit.Caption := Settings.GetFilesSizeUnit(Messages.Strings[msgByteString]);

end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ApplyLanguage;
var
  LangFile: TFileName;
  i: Integer;
  newMenuItem: TTBXItem;
begin
  with Settings.rOptions do
  begin
    if rLanguage.Language = '?' then
      ActionLanguage.Execute;
    LangFile := Settings.GetLanguageFile;
    if (LangFile <> '') and (rLanguage.Language <> Settings.Language) and FileExists(LangFile) then
    begin
      Translator.SetLanguageFile(LangFile);
      Self.Font.Name := Graphics.DefFontData.Name;
      Self.Font.Charset := Graphics.DefFontData.Charset;
      Translator.Translate(MainWindow);
      Translator.Translate(FrmMovie);
      NumberWin.Translate;
      InputWin.Translate;
      MessageWin.Translate;
    end;
    Settings.Language := rLanguage.Language;
    strHelpFile := ChangeFileExt(LangFile, '.chm');
    if not FileExists(strHelpFile) then
      strHelpFile := strDirApp + strDirLanguages + strFileHelp;
    FCatalogFile.MessageSaveQuery := Messages.Strings[msgUnsavedFile];
    FrmMovie.LSizeUnit.Caption := Settings.GetFilesSizeUnit(Messages.Strings[msgByteString]);
  end;
  strFieldPicture := Messages.Strings[msgPicture];
  RebuildSearchMenu;
  { --- Group by --- }
  with MnuTlsGrp do
    while Count > 2 do
      Delete(2);
  with strFields do
    for i := 0 to fieldCount-1 do
      if (i in GroupByFields) then
      begin
        newMenuItem := TTBXItem.Create(MnuTlsGrp);
        MnuTlsGrp.Add(newMenuItem);
        with newMenuItem do
        begin
          Caption := Strings[i];
          Hint := Format(Messages.Strings[msgGroupBy], [Strings[i]]);
          Tag := i;
          OnClick := ActionGroupNoneExecute;
          GroupIndex := 1;
        end;
      end;
  SetGroupField(FGroupField);
  { --- Sort by --- }
  with MnuTlsSort do
    while Count > 3 do
      Delete(3);
  with strFields do
    for i := 0 to fieldCount-1 do
      if (i in SortByFields) then
      begin
        newMenuItem := TTBXItem.Create(MnuTlsSort);
        MnuTlsSort.Add(newMenuItem);
        with newMenuItem do
        begin
          Caption := Strings[i];
          Hint := Format(Messages.Strings[msgSortBy], [Strings[i]]);
          Tag := i;
          OnClick := ActionSortExecute;
          GroupIndex := 2;
        end;
      end;
  SetSortField(Abs(FSortField) - 1, FSortField < 0);
  { --- Find --- }
  i := cbxField.ItemIndex;
  if i = -1 then
    i := Settings.rMain.FindField + 1;
  cbxField.Items.Assign(Fields.Strings);
  cbxField.Items.Insert(0, Messages.Strings[msgFindFieldAll]);
  cbxField.ItemIndex := i;
  { --- HTMLViewer --- }
  PreGenerateHTMLTemplate;
  { --- ListView1 --- }
  RefreshMovieList();
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.RebuildSearchMenu;
var
  newMenuItem: TTBXItem;
  i: Integer;
begin
  MnuMovSch.Clear;
  with Settings.rOptions.rMovieInformation.SearchSites do
    for i := 0 to Count-1 do
    begin
      if Pos('-', Strings[i]) = 1 then
        MnuMovSch.Add(TTBXSeparatorItem.Create(MnuMovSch))
      else
      begin
        newMenuItem := TTBXItem.Create(MnuMovSch);
        MnuMovSch.Add(newMenuItem);
        with newMenuItem do
        begin
          Caption := Names[i];
          Hint := Format(Messages.Strings[msgSearchInfo], [Names[i]]);
          Tag := i;
          OnClick := SearchInternetClick;
        end;
      end; // if-else
    end;
end;

{-------------------------------------------------------------------------------
   Actions
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileNewExecute(Sender: TObject);
begin
  SetSaveFilter;
  FCatalogFile.New;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileOpenExecute(Sender: TObject);
begin
  SetSaveFilter;
  with FCatalogFile do
  begin
    OpenDialog.InitialDir := Settings.rOptions.rFolders[fuCatalogs].Value;
    if Open then
    begin
      Settings.rOptions.rFolders[fuCatalogs].Value := ExtractFilePath(CurrentFile);
      UpdateMRU(CurrentFile);
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetSaveFilter;
var
  ext: string;
begin
  if FCatalogFile.CurrentFile <> '' then
    ext := ExtractFileExt(FCatalogFile.CurrentFile)
  else
    ext := extCatalog[extAMC];
  if ext = extCatalog[extAMC] then
    FCatalogFile.SaveDialog.FilterIndex := DialogCatalogFilterAMC
  else
  if ext = extCatalog[extXML] then
    FCatalogFile.SaveDialog.FilterIndex := DialogCatalogFilterXML;
  Delete(ext, 1, 1);
  with FCatalogFile.SaveDialog do
  begin
    DefaultExt := ext;
    InitialDir := Settings.rOptions.rFolders[fuCatalogs].Value;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileSaveExecute(Sender: TObject);
begin
  SetSaveFilter;
  FCatalogFile.Save;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileSaveAsExecute(Sender: TObject);
begin
  SetSaveFilter;
  with FCatalogFile do
  begin
    if SaveAs then
    begin
      Settings.rOptions.rFolders[fuCatalogs].Value := ExtractFilePath(CurrentFile);
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileExportExecute(Sender: TObject);
begin
  SaveCurrentItem;
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TExportWin, ExportWin);
    try
      ImageListHot.GetIcon(Ord(ICON_FILEEXPORT), ExportWin.Icon);
      StoreSelectedState;
      ExportWin.Execute(MovieList, FCatalogFile.CurrentFile);
    finally
      ExportWin.Release;
      ExportWin := nil;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFilePrintExecute(Sender: TObject);
begin
  SaveCurrentItem;
  with ProgressWin do
  begin
    Maximum := 1;
    Status := Messages.Strings[msgPrintLoading];
    Progress := Messages.Strings[msgPrintSearching];
    Execute(Self);
    try
      Application.ProcessMessages;
      printform.Init_FR;
      IntProgress := 1;
    finally
      Close;
    end;
  end;
  try
    Application.CreateForm(TPrintWin, PrintWin);
    try
      ImageListHot.GetIcon(Ord(ICON_FILEPRINT), PrintWin.Icon);
      StoreSelectedState;
      PrintWin.Execute(FCatalogFile.CurrentFile, MovieList);
    finally
      PrintWin.Release;
      PrintWin := nil;
    end;
  finally
    printform.Final_FR;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieAddExecute(Sender: TObject);
begin
  ThumbsViewerStop;
  with Settings.rOptions.rMovieInformation do
    if(NumberWin.Execute(MovieList, true, not AskNumber, AskNumber) = mrOk) then
    begin
      if NumberWin.CBDoNotAsk.checked then
      begin
        AskNumber := false;
      end;
      NewItem;
      with FrmMovie.EOriginalTitle do
        if CanFocus then
          SetFocus;
      if Sender <> nil then
      begin
        if AddFiles then
          ActionMovieImportFiles.Execute;
        if AddScript then
          ActionMovieImportScript.Execute;
      end;
    end;
  ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieNumberExecute(Sender: TObject);
var
  CurMovie: TMovie;
  Existing: TMovie;
begin
  CurMovie := nil;
  if (ListView1.Selected <> nil) and (ListView1.Selected.Selected) then
    CurMovie := TMovie(ListView1.Selected.Data);
  if CurMovie <> nil then
  begin
    if NumberWin.Execute(MovieList, False, False, True, CurMovie.iNumber) = mrOk then
    begin
      case NumberWin.grpNotUnique.ItemIndex of
        1:
          begin
            Existing := MovieList.Find(NumberWin.ENumber.AsInteger);
            if Existing <> nil then
            begin
              if Settings.rOptions.rMovieInformation.FirstAvailable then
                Existing.iNumber := MovieList.FirstFreeNumber
              else
                Existing.iNumber := MovieList.MaxNumber + 1; // + 1 !
            end;
          end;
        2:
          begin
            MovieList.ShiftNumbers(NumberWin.ENumber.AsInteger, CurMovie);
          end;
      end;
      CurMovie.iNumber := NumberWin.ENumber.AsInteger;
      FCatalogFile.Modified := True;
      RefreshMovieList;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieUndoExecute(Sender: TObject);
var
  DoNotAsk: boolean;
begin
  if (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil) and
    (ListView1.Selected.Data <> nil) and (MovieSave <> nil) then
  begin
    if Settings.rOptions.rMovieList.ConfirmUndo then
    begin
      DoNotAsk := false;
      if MessageWin.Execute(Messages.Strings[msgUndo], mtConfirmation, DoNotAsk,
                            Messages.Strings[msgDoNotConfirm], [mbYes,mbNo]) = 1 then
        Settings.rOptions.rMovieList.ConfirmUndo := not DoNotAsk
      else
        exit;
    end;
    ThumbsViewerStop;
    TMovie(Listview1.Selected.Data).Assign(MovieSave);
    OpenItem(Listview1.Selected);
    UpdateItem(ListView1.Selected);
    ThumbsViewerStart;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieDeleteExecute(Sender: TObject);
var
  i,j: Integer;
  AllClicked: Boolean;
  ListDelete: TStringList;
  KeyMovie: string;
begin
  AllClicked := False;
  FPreviousItem := nil;
  with ListView1 do
  begin
    if SelectedCount > 0 then
    begin
      with Items do
      begin
        try
          ListDelete := TStringList.Create;
          ListDelete.Sorted := True;
          for i := 0 to Count-1 do
            with Item[i] do
            begin
              if (Selected) then
                if (Data <> nil) then
                begin
                  KeyMovie := Format('%p', [Data]);
                  if ListDelete.IndexOf(KeyMovie) = -1 then
                    ListDelete.AddObject(KeyMovie, Data);
                end else
                begin
                  for j := 0 to Items[i].ChildrenCount-1 do
                  begin
                    KeyMovie := Format('%p', [Items[i].Children[j].Data]);
                    if ListDelete.IndexOf(KeyMovie) = -1 then
                      ListDelete.AddObject(KeyMovie, Items[i].Children[j].Data);
                  end;
                end;
            end;
          ThumbsViewerStop;
          BeginUpdate;
          for i := 0 to ListDelete.Count-1 do
          begin
            case Self.DeleteItem(TMovie(ListDelete.Objects[i]), not AllClicked, True) of
              2:  AllClicked := True;
              4:  break;
            end;
            if not AllClicked then
            begin
              EndUpdate;
              SetStatus;
              BeginUpdate;
            end;
          end;
          ListDelete.Free;
          // Fix bug in selected item after delete
          if SelectedCount = 1 then
            for i := 0 to Count-1 do
              if Item[i].Selected then
                Selected := Item[i];
          if SelectedCount = 0 then
            ClearItem;
        finally
          EndUpdate;
          ThumbsViewerStart;
        end;
      end; // end with Items
    end; // if SelectedCount > 0
  end; // with ListView1

  ListView1AfterSelectionChange(Sender);
  ListView1EnsureVisiblePlus;
  SetStatus;
  FCatalogFile.Modified := True;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionHelpAboutExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TAboutWin, AboutWin);
    try
      ImageListHot.GetIcon(Ord(ICON_ABOUT), AboutWin.Icon);
      AboutWin.ShowModal;
    finally
      AboutWin.Release;
      AboutWin := nil;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionHelpVersionExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TAboutWin, AboutWin);
    try
      AboutWin.ShowVersions;
    finally
      AboutWin.Release;
      AboutWin := nil;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionExitExecute(Sender: TObject);
begin
  Close;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieImportFilesExecute(Sender: TObject);
var
  s: string;
  i: Integer;
begin
  ThumbsViewerStop;
  with TOpenDialog.Create(Self) do
    try
      InitialDir := Settings.rOptions.rFolders[fuGetFromFiles].Value;
      FileName := '';
      Title := Messages.Strings[msgSelectFilesInfo];
      Options := DialogOpenOptions + [ofAllowMultiSelect];
      s := '';
      for i := Low(extVideo) to High(extVideo) do
      begin
        if s <> '' then
          s := s + ';';
        s := Format('%s*%s', [s, extVideo[i]]); 
      end;
      Filter := Format(DialogGetInfoFilter, [s]);
      if Execute then
      begin
        Settings.rOptions.rFolders[fuGetFromFiles].Value := ExtractFilePath(FileName);
        DragDropFilesDrop(sender, point(0,0), TStringList(Files));
      end;
  finally
    Free;
  end;
  ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieImportCDExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieImportScriptExecute(Sender: TObject);
begin
  if (ListView1.SelectedCount = 0) then
    ActionMovieAddExecute(nil);
  if (ListView1.SelectedCount > 0) then
    ActionToolsScriptingExecute(ActionMovieImportScript);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionToolsScriptingExecute(Sender: TObject);
var
  WorkMode: TScriptWorkMode;
begin
  if Sender = ActionMovieImportScript then
    WorkMode := swmGetInfo
  else
    WorkMode := swmScripting;
  StoreSelectedState;
  SaveCurrentItem;
  FPreviousItem := nil;
  Screen.Cursor := crHourGlass;
  ThumbsViewerStop;
  try
    GetScriptWin := TGetScriptWin.Create(Self, WorkMode);
    with GetScriptWin do
      try
        ImageListHot.GetIcon(Ord(ICON_MOVIEIMPORTSCRIPT), Icon);
        if GetScriptWin.Execute(FCatalogFile.CurrentFile, MovieList) = mrOk then
        begin
          FCatalogFile.Modified := True;
          RefreshMovieList(False);
          LoadSelectedState;
          ListView1EnsureVisiblePlus;
        end;
      finally
        Release;
        GetScriptWin := nil;
      end;
  finally
    Screen.Cursor := crDefault;
    ThumbsViewerStart;
  end;
  FPreviousItem := ListView1.Selected;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieFindExecute(Sender: TObject);
begin
  ToolbarFind.Visible := ActionMovieFind.Checked;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieCopyExecute(Sender: TObject);
var
  TempMovie: TMovie;
  DataHandle: THandle;
begin
  with ListView1 do
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data <> nil) then
        begin
          TMovie(Data).Lock;
          TempMovie := TMovie.Create;
          try
            FrmMovie.SaveToObject(TempMovie, True);
            with TMovie(Data) do
            begin
              if strPicture <> '' then
              begin
                TempMovie.strPicture := strPicture;
                if Picture <> nil then
                begin
                  TempMovie.Picture := TMemoryStream.Create;
                  Picture.Seek(0, soFromBeginning);
                  TempMovie.Picture.CopyFrom(Picture, Picture.Size);
                end;
              end;
            end;
            try
              DataHandle := TempMovie.SaveToMemory;
              try
                ClipBoard.SetAsHandle(ClipboardFormat, DataHandle);
              finally
                TempMovie.FreeMemory(DataHandle, True);
              end;
            except
              on e:Exception do
              begin
                MessageWin.Execute('Copy to clipboard - Error: '+e.Message, mtError, [mbOk]);
              end;
            end;
          finally
            TempMovie.Free;
            TMovie(Data).Unlock;
          end;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMoviePasteExecute(Sender: TObject);
var
  TempMovie: TMovie;
  DataHandle: THandle;
begin
  with ListView1 do
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data <> nil) then
        begin
          ThumbsViewerStop;
          if(MovieSave = nil) then
          begin
            MovieSave := TMovie.Create;
            MovieSave.Assign(Data);
          end;
          TempMovie := TMovie.Create;
          try
            DataHandle := ClipBoard.GetAsHandle(ClipboardFormat);
            if DataHandle <> 0 then
            begin
              TempMovie.LoadFromMemory(DataHandle);
              if TempMovie.strPicture <> '' then
              begin
                with TMovie(Data) do
                begin
                  Picture.Free;
                  Picture := nil;
                  strPicture := TempMovie.strPicture;
                  if TempMovie.Picture <> nil then
                  begin
                    Picture := TMemoryStream.Create;
                    TempMovie.Picture.Seek(0, soFromBeginning);
                    Picture.CopyFrom(TempMovie.Picture, TempMovie.Picture.Size);
                  end;
                  if (_thumb <> nil) then
                    FreeAndNil(_thumb);
                  _thumbError := 0;
                end;
              end;
              OpenItem(TempMovie);
              FrmMovie.Modified := True;
              FCatalogFile.Modified := True;
              ListView1UpdateItem;
            end;
          finally
            TempMovie.Free;
            ThumbsViewerStart;
          end;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieRenumberExecute(Sender: TObject);
var
  Refresh: Boolean;
begin
  ThumbsViewerStop;
  Refresh := False;
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TRenumberWin, RenumberWin);
    try
      ImageListHot.GetIcon(Ord(ICON_RENUMBER), RenumberWin.Icon);
      if RenumberWin.Execute(MovieList) = mrOk then
      begin
        FCatalogFile.Modified := True;
        RefreshMovieList;
        Refresh := True;
      end;
    finally
      RenumberWin.Release;
      RenumberWin := nil;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
  if not Refresh then
    ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFileImportExecute(Sender: TObject);
begin
  SaveCurrentItem;
  ThumbsViewerStop;
  with TImportWin2.Create(Self) do
    try
      ImageListHot.GetIcon(Ord(ICON_FILEIMPORT), Icon);
      if Execute(FCatalogFile.CurrentFile, MovieList) then
      begin
        RefreshMovieList(True);
        FCatalogFile.Modified := True;
      end;
    finally
      Release;
    end;
  ThumbsViewerStart;
{
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TImportWin, ImportWin);
    try
      ImageListHot.GetIcon(ICON_FILEIMPORT, ImportWin.Icon);
      if ImportWin.Execute(FCatalogFile.CurrentFile) = mrOk then
      begin
        SaveCurrentItem;
        with ProgressWin do
        begin
          IntProgress := 0;
          Status := Format(Messages.Strings[msgImporting],[ExtractFileName(ImportWin.edtSourceFile.Text)]);
          Execute(Self);
        end;
        try
          Application.ProcessMessages;
          ProgressWin.Maximum := 1;
          MovieList.Add(ImportWin.GetMovies, Settings.rImport.AllowDupNumbers);
          FCatalogFile.Modified := True;
          ProgressWin.IntProgress := 1;
        finally
          ProgressWin.Close;
        end;
      end;
    finally
      ImportWin.Release;
      ImportWin := nil;
    end;
    RefreshMovieList;
  finally
    Screen.Cursor := crDefault;
  end;
}
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionHelpIndexExecute(Sender: TObject);
begin
  LaunchHelp(self);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionLoanExecute(Sender: TObject);
var
  FileModified: Boolean;
begin
  ActionLoan.Enabled := False;
  SaveCurrentItem;
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TLoanWin, LoanWin);
    try
      ImageListHot.GetIcon(Ord(ICON_LOANS), LoanWin.Icon);
      LoanWin.Execute(MovieList, ExtractFileName(FCatalogFile.CurrentFile), FrmMovie.EBorrower.Items, FileModified);
      with Settings.rOptions.rMovieInformation.rCombo[ddlBorrowers] do
        if not UseCatalogValues then
          Contents.Assign(FrmMovie.EBorrower.Items);
    finally
      LoanWin.Release;
      LoanWin := nil;
    end;
    FCatalogFile.Modified := FileModified or FCatalogFile.Modified;
  finally
    Screen.Cursor := crDefault;
  end;
  if FileModified then
    RefreshMovieList;
  ActionLoan.Enabled := True;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionGroupNoneExecute(Sender: TObject);
begin
  if Sender = ActionGroupNone then
  begin
    MnuGrpNon.Checked := True;
    FGroupField := -1;
  end else
  if Sender is TTBXItem then
  begin
    with (Sender as TTBXItem) do
    begin
      Checked := True;
      FGroupField := Tag;
    end;
  end;
  RefreshMovieList;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetGroupField(Field: Integer);
var
  i: Integer;
begin
  if not (Field in GroupByFields) then
    Field := -1;
  FGroupField := Field;
  with MnuTlsGrp do
    for i := 0 to Count-1 do
      with Items[i] do
        if (Tag = Field) then
        begin
          Checked := True;
          Break;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionSortExecute(Sender: TObject);
begin
  if (Sender = ActionSortAscend) or (Sender = ActionSortDescend) then
    SetSortField(Abs(FSortField) - 1, Sender = ActionSortDescend)
  else if Sender is TTBXItem then
    SetSortField(TTBXItem(Sender).Tag, MnuTlsSortDescend.Checked);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetSortField(Field: Integer; Descend: Boolean);
var
  i: Integer;
  section: TElHeaderSection;
begin
  if not (Field in SortByFields) then
    Field := fieldNumber;
   FSortField := Field + 1;
   if Descend then
     FSortField := FSortField * -1;

  if Descend then
    MnuTlsSortDescend.Checked := True
  else
    MnuTlsSortAscend.Checked := True;

  with MnuTlsSort do
    for i := 3 to Count-1 do
      with Items[i] do
        if (Tag = Field) then
        begin
          Checked := True;
          Break;
        end;

  section := nil;
  with ListView1 do
  with HeaderSections do
  begin
    if ActionToolsGrid.Checked then
    begin
      for i:=2 to Count-1 do
        if StrToInt(Item[i].FieldName) = Field then
        begin
          section := Item[i];
          break;
        end;
    end else
    begin
      if Field = fieldNumber then
        section := Item[0]
      else if Field = fieldFormattedTitle then
        section := Item[1]
      else
        section := Item[2];
    end;

    if section <> nil then
    begin
      ListView1.SortSection := section.Index;
      if Descend then
        Item[SortSection].SortMode := hsmDescend
      else
        Item[SortSection].SortMode := hsmAscend;
      ListView1.Sort(true);
    end;
  end;
  ListView1EnsureVisiblePlus;
  ThumbsViewerStart;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionToolsGridExecute(Sender: TObject);
begin
  SaveCurrentItem;
  FPreviousItem := nil;
  ActionToolsGrid.Checked := not ActionToolsGrid.Checked;
  RefreshMovieList;
end;
{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionRefreshExecute(Sender: TObject);
begin
  RefreshMovieList;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionOptionsExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TOptionsWin, OptionsWin);
    try
      ImageListHot.GetIcon(Ord(ICON_OPTIONS), OptionsWin.Icon);
      StoreSelectedState; // Need to be done here !
      if OptionsWin.Execute(ToolbarMain, ImageListHot) = mrOk then
      begin
        ApplyOptions;
        // Reload HTMLTemplate
        PreGenerateHTMLTemplate;
        // Reload ListView1
        RefreshMovieList(False);
        LoadSelectedState;
        ListView1EnsureVisiblePlus;
      end;
    finally
      OptionsWin.Release;
      OptionsWin := nil;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionLanguageExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  with TLanguageWin.Create(Self) do
    try
      ImageListHot.GetIcon(Ord(ICON_LANGUAGES), Icon);
      if Execute then
        ApplyLanguage;
    finally
      Screen.Cursor := crDefault;
      Release;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionDisplayPictureToolbarExecute(Sender: TObject);
begin
  ToolbarPicture.Visible := ActionDisplayPictureToolbar.Checked;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionDisplayMainToolbarExecute(Sender: TObject);
begin
  ToolbarMain.Visible := ActionDisplayMainToolbar.Checked;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionDisplayStatusBarExecute(Sender: TObject);
begin
  StatusBar1.Visible := ActionDisplayStatusBar.Checked;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieStatsExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    SaveCurrentItem;
    StoreSelectedState;
    Application.CreateForm(TStatsWin, StatsWin);
    try
      ImageListHot.GetIcon(Ord(ICON_STATISTICS), StatsWin.Icon);
      StatsWin.Execute(MovieList, FCatalogFile.CurrentFile);
    finally
      FreeAndNil(StatsWin);
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuFileExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuMovieExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuToolsExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuHelpExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuPictureExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuGetExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuGroupExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMenuSortExecute(Sender: TObject);
begin
  //
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieSearchExecute(Sender: TObject);
begin
//
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFilePropertiesExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    Application.CreateForm(TPropertiesWin, PropertiesWin);
    with PropertiesWin do
      try
        ImageListHot.GetIcon(Ord(ICON_FILEPROPERTIES), Icon);
        if Execute(FCatalogFile.CurrentFile, MovieList) then
          FCatalogFile.Modified := True;
      finally
        Release;
        PropertiesWin := nil;
      end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieCheckExecute(Sender: TObject);
var
  i:integer;
begin
  ListView1.Items.BeginUpdate;
  with ListView1 do
    with Items do
      for i := 0 to Count-1 do
      begin
        if Item[i].Selected then
        begin
          Item[i].Checked := True;
          OnItemChecked(Sender, Item[i]);
        end;
      end;
  ListView1.Items.EndUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieUncheckExecute(Sender: TObject);
var
  i:integer;
begin
  ListView1.Items.BeginUpdate;
  with ListView1 do
    with Items do
      for i := 0 to Count-1 do
      begin
        if Item[i].Selected then
        begin
          Item[i].Checked := False;
          OnItemChecked(Sender, Item[i]);
        end;
      end;
  ListView1.Items.EndUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieSelGroupExecute(Sender: TObject);
var
  i: Integer;
begin
  with ListView1 do
  begin
    Items.BeginUpdate;
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data = nil) then
        begin
          Selected := False;
          if not Expanded then
            Expand(True);
          for i := 0 to Count-1 do
          begin
            if i = 0 then
            begin
              ListView1.ItemFocused := Item[i];
              ListView1.Selected := Item[i];
            end;
            Item[i].Selected := True;
          end;
        end;
      Items.EndUpdate;
      ListView1AfterSelectionChange(Sender);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieSelCheckExecute(Sender: TObject);
var
  i: Integer;
begin
  ListView1.DeselectAll;
  with ListView1 do
    with Items do
      for i := 0 to Count-1 do
        Item[i].Selected := (Item[i].Data <> nil) and (Item[i].Checked);
  ListView1.Selected := ListView1.GetNextSelected(nil);
  ListView1AfterSelectionChange(ListView1);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieSelUncheckExecute(Sender: TObject);
var
  i: Integer;
begin
  ListView1.DeselectAll;
  with ListView1 do
    with Items do
      for i := 0 to Count-1 do
        Item[i].Selected := (Item[i].Data <> nil) and (not Item[i].Checked);
  ListView1.Selected := ListView1.GetNextSelected(nil);
  ListView1AfterSelectionChange(ListView1);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieNextExecute(Sender: TObject);
var
  prevFocused: TWinControl;
begin
  with ListView1 do
  begin
    if Items.Count > 0 then
    begin
      prevFocused := ActiveControl;
      if (prevFocused <> nil) {and not (prevFocused is TElTreeView)} then
      begin
        MultiSelect := False;
        if Selected = nil then
          Selected := Items.Item[0]
        else
        begin
          //ListView1UpdateItem; <-- Check in SaveCurrentItem
          Selected := Selected.GetNext;
        end;
        if (Selected <> nil) and (Selected.Data = nil) then
          Selected := Selected.GetNext;
        ListView1EnsureVisiblePlus;
        MultiSelect := True;
        ListView1AfterSelectionChange(Sender);
        if prevFocused.Enabled then
          FocusControl(prevFocused);
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMoviePreviousExecute(Sender: TObject);
var
  prevFocused: TWinControl;
begin
  with ListView1 do
  begin
    if Items.Count > 0 then
    begin
      prevFocused := ActiveControl;
      if (prevFocused <> nil) {and not (prevFocused is TElTreeView)} then
      begin
        MultiSelect := False;
        if Selected = nil then
          Selected := Items.Item[Items.Count-1]
        else
        begin
          //ListView1UpdateItem; <-- Check in SaveCurrentItem
          Selected := Selected.GetPrev;
        end;
        if (Selected <> nil) and (Selected.Data = nil) then
          Selected := Selected.GetPrev;
        ListView1EnsureVisiblePlus;
        MultiSelect := True;
        ListView1AfterSelectionChange(Sender);
        if prevFocused.Enabled then
          FocusControl(prevFocused);
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionURLOpenExecute(Sender: TObject);
begin
  with FrmMovie.EURL do
    if Text <> '' then
      LaunchProg(Text);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionURLBrowseExecute(Sender: TObject);
begin
  with TOpenDialog.Create(Self) do
    try
      InitialDir := Settings.rOptions.rFolders[fuGetFromFiles].Value;
      FileName := '';
      Title := Messages.Strings[msgSelectFileURL];
      Options := DialogOpenOptions;
      Filter := DialogAnyFileFilter;
      if Execute then
      begin
        Settings.rOptions.rFolders[fuGetFromFiles].Value := ExtractFilePath(FileName);
        FrmMovie.EURL.Text := FileName;
      end;
  finally
    Free;
  end;
end;

{-------------------------------------------------------------------------------
   Events
-------------------------------------------------------------------------------}

procedure TMainWindow.OnMovieFieldChange(Sender: TObject);
begin
  FCatalogFile.Modified := True;
{ We don't update tree item here because is too slow when there are many items in tree.
  The item is updated later in this cases (call ListView1UpdateItem procedure) :
  - The user move the mouse on the tree
  - The user move the mouse on the left side of FrmMovie (X < 90).
  - The user press ENTER or ESCAPE in a field to force update.
    - ENTER is not take account for fields Description and Comments
  - The user press a header column of tree to sort items.
  - The user press stretch list button
  - The user paste a movie on the selected movie
  - The user cancel modifications
  - Others : Check in SaveCurrentItem function }
end;

procedure TMainWindow.ListView1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if(ListView1.CanFocus) then
    ListView1.SetFocus;
  ListView1UpdateItem;
end;

procedure TMainWindow.FrmMovieMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if (X < 90) then
    ListView1UpdateItem;
end;

procedure TMainWindow.MovieFieldKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) or ((Key = VK_RETURN) and (Sender <> FrmMovie.EDescription) and
    (Sender <> FrmMovie.EComments)) then
    ListView1UpdateItem;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.MovieFieldKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = Ord('A')) and (Shift = [ssCtrl]) then
  begin
    if Sender is TEdit then
      TEdit(Sender).SelectAll
    else
    if Sender is TMemo then
      TMemo(Sender).SelectAll
    else
    if Sender is TComboBox then
      TComboBox(Sender).SelectAll;
  end
  else
  if (Key = VK_RETURN) and (Sender = FrmMovie.EURL) then
    ActionURLOpen.Execute;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FormShow(Sender: TObject);
var
  s, FileToOpen: string;
begin
  SplashWin.ProgressBar1.Position := 60;
  LoadOptions;
  SplashWin.ProgressBar1.Position := 60;
  ApplyLanguage;
  SplashWin.ProgressBar1.Position := 70;
  Application.ProcessMessages;
  SplashWin.Release;
  SplashWin := nil;
  FCatalogFile.Modified := False;
  with Settings.rOptions.rFiles do
    if (AutoLoad) and (AutoLoadFile <> '') then
      FileToOpen := AutoLoadFile
    else
      FileToOpen := '';
  if ParamCount > 0 then
  begin
    s := ParamStr(1);
    if (s <> '') and (s[1] <> '/') then
      FileToOpen := s;
  end;
  if FileToOpen <> '' then
    FCatalogFile.Open(ExpandFileName(FileToOpen))
  else
    FCatalogFile.New;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FormCreate(Sender: TObject);
var
  i : integer;
begin
  Font.Name := Graphics.DefFontData.Name;
  Font.Charset := Graphics.DefFontData.Charset;
  {$IFDEF DISABLETHEMES}
  TBXSwitcher1.EnableXPStyles := False;
  TBXSwitcher1.FlatMenuStyle := fmsDisable;
  ToolbarFind.Color := clBtnFace;
  ToolbarMain.Color := clBtnFace;
  ToolbarMenu.Color := clBtnFace;
  ToolbarPicture.Color := clBtnFace;
  ToolbarPictureWindow.Color := clBtnFace;
  DockBottomList.Color := clBtnFace;
  DockImageLeft.Color := clBtnFace;
  DockImageTop.Color := clBtnFace;
  DockMainBottom.Color := clBtnFace;
  DockMainLeft.Color := clBtnFace;
  DockMainTop.Color := clBtnFace;
  DockRightInfo.Color := clBtnFace;
  {$ENDIF}

  DragDropFiles := TJvDragDrop.Create(Self);
  DragDropFiles.OnDrop := DragDropFilesDrop;
  DragDropFiles.AcceptDrag := True;

  FPreviousItem := nil;
  MovieList := nil;
  MovieSave := nil;
  FGroupField := -1;
  GroupItemEmpty := nil;
  MoviesVisible := 0;
  
  FCatalogFile := TFileManager.Create(Self);
  FCatalogFile.OnFileChange := OnFileChange;
  FCatalogFile.OnFileModified := OnFileModified;
  FCatalogFile.OnNewFile := OnNewFile;
  FCatalogFile.OnOpenFile := OnOpenFile;
  FCatalogFile.OnSaveFile := OnSaveFile;
  with FCatalogFile do
  begin
    with OpenDialog do
    begin
      Filter := DialogCatalogFilter;
      Options := DialogOpenOptions;
    end;
    with SaveDialog do
    begin
      Filter := DialogCatalogSaveFilter;
      Options := DialogSaveOptions;
      OnTypeChange := Self.OnDialogTypeChange;
    end;
  end;

  ActionFileNew.ImageIndex := Ord(ICON_FILENEW);
  ActionFileOpen.ImageIndex := Ord(ICON_FILEOPEN);
  ActionFileSave.ImageIndex := Ord(ICON_FILESAVE);
  ActionFileSaveAs.ImageIndex := Ord(ICON_FILESAVEAS);
  ActionFileImport.ImageIndex := Ord(ICON_FILEIMPORT);
  ActionFileExport.ImageIndex := Ord(ICON_FILEEXPORT);
  ActionFilePrint.ImageIndex := Ord(ICON_FILEPRINT);
  ActionFileProperties.ImageIndex := Ord(ICON_FILEPROPERTIES);
  ActionExit.ImageIndex := Ord(ICON_EXIT);
  ActionMovieAdd.ImageIndex := Ord(ICON_MOVIEADD);
  ActionMovieNumber.ImageIndex := Ord(ICON_MOVIENUMBER);
  ActionMovieCopy.ImageIndex := Ord(ICON_MOVIECOPY);
  ActionMoviePaste.ImageIndex := Ord(ICON_MOVIEPASTE);
  ActionMovieDelete.ImageIndex := Ord(ICON_MOVIEDELETE);
  ActionMovieFind.ImageIndex := Ord(ICON_MOVIEFIND);
  ActionMovieUndo.ImageIndex := Ord(ICON_MOVIEUNDO);
  ActionMoviePictureShow.ImageIndex := Ord(ICON_MOVIEPICTURE);
  ActionMenuPicture.ImageIndex := Ord(ICON_MOVIEPICTURE);
  ActionMenuGet.ImageIndex := Ord(ICON_MOVIEIMPORT);
  ActionMovieImportFiles.ImageIndex := Ord(ICON_MOVIEIMPORTFILES);
  ActionMovieImportCD.ImageIndex := Ord(ICON_MOVIEIMPORTCD);
  ActionMovieImportScript.ImageIndex := Ord(ICON_MOVIEIMPORTSCRIPT);
  ActionMovieSearch.ImageIndex := Ord(ICON_MOVIESEARCH);
  ActionMovieStats.ImageIndex := Ord(ICON_STATISTICS);
  ActionHelpAbout.ImageIndex := Ord(ICON_ABOUT);
  ActionHelpIndex.ImageIndex := Ord(ICON_HELP);
  ActionLoan.ImageIndex := Ord(ICON_LOANS);
  ActionToolsScripting.ImageIndex := Ord(ICON_SCRIPTING);
  ActionToolsGrid.ImageIndex := Ord(ICON_GRIDMODE);
  ActionMovieRenumber.ImageIndex := Ord(ICON_RENUMBER);
  ActionOptions.ImageIndex := Ord(ICON_OPTIONS);
  ActionLanguage.ImageIndex := Ord(ICON_LANGUAGES);
  ActionMenuGroup.ImageIndex := Ord(ICON_GROUP);
  ActionMenuSort.ImageIndex := Ord(ICON_VIEWDETAIL);
  ActionRefresh.ImageIndex := Ord(ICON_REFRESH);
  ActionPicSelect.ImageIndex := Ord(ICON_PICTUREOPEN);
  ActionPicDelete.ImageIndex := Ord(ICON_PICTUREDELETE);
  ActionPicUndock.ImageIndex := Ord(ICON_PICTUREUNDOCK);
  ActionPicSaveAs.ImageIndex := Ord(ICON_PICTURESAVE);
  ActionPicCopy.ImageIndex := Ord(ICON_PICTURECOPY);
  ActionMovieRandom.ImageIndex := Ord(ICON_RANDOM);
  ActionStretchList.ImageIndex := Ord(ICON_MOVERIGHTALL);
  ActionDisplayHTML.ImageIndex := Ord(ICON_HTML);
  ActionDisplayThumbnails.ImageIndex := Ord(ICON_VIEWLIST);

  Settings.version := StrVersion;
  Application.OnHelp := OnAppHelp;
  FrmMovie.OnFieldChange := Self.OnMovieFieldChange;

  MStream := TMemoryStream.Create;
  HTMLTemplatePreGenerated := TStringList.Create;
  HTMLTemplatePath := '';

  for i := 0 to High(MStreamAppr) do
    if FileExists(strDirApp+'Templates\appr'+IntToStr(i)+'.gif') then
    begin
      MStreamAppr[i] := TMemoryStream.Create;
      MStreamAppr[i].LoadFromFile(strDirApp+'Templates\appr'+IntToStr(i)+'.gif');
    end else
      MStreamAppr[i] := nil;

  for i := 0 to High(MStreamAppr10) do
    if FileExists(strDirApp+'Templates\appr10_'+IntToStr(i)+'.gif') then
    begin
      MStreamAppr10[i] := TMemoryStream.Create;
      MStreamAppr10[i].LoadFromFile(strDirApp+'Templates\appr10_'+IntToStr(i)+'.gif');
    end else
      MStreamAppr10[i] := nil;

  {ThumbsViewer}
  ThumbJPEG := TJpegImageFix.Create;
  ThumbJPEG.CompressionQuality := 80;
  ThumbJPEG.Performance := jpBestSpeed;
  ThumbGIF := TJvGIFImage.Create;
  ThumbPNG := TPNGObject.Create;
  // Max thumbnail size...
  ThumbSizeW := Trunc(255*0.75);
  ThumbSizeH := 255;
  ThumbsViewer.CellWidth := ThumbSizeW + 10;
  ThumbsViewer.CellHeight := ThumbSizeH + 30;
  CellJpeg := TJpegImageFix.Create;
  CellJPEG.Performance := jpBestSpeed;
  GenCellColors;
  CellStyle := -1;
  PoolSize := 0;
  MaxPool := Round(((Screen.Width * Screen.Height) * 3) * 1.5);
  ThumbsPool := TList.Create;
  {End ThumbsViewer}
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FormDestroy(Sender: TObject);
var
  i: integer;
begin
  {ThumbsViewer}
  ThumbsViewerStop;
  ThumbJPEG.Free;
  ThumbGIF.Free;
  ThumbPNG.Free;
  CellJpeg.Free;
  ThumbsPool.Free;
  {End ThumbsViewer}

  MovieList.Free;
  MovieSave.Free;
  FCatalogFile.Free;
  DragDropFiles.Free;
  MStream.Free;
  for i := 0 to High(MStreamAppr) do
    MStreamAppr[i].Free;
  for i := 0 to High(MStreamAppr10) do
    MStreamAppr10[i].Free;
  HTMLTemplatePreGenerated.Free;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveOptions;
  ListView1.DeselectAll;
  MovieSelected;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  SetSaveFilter;
  with Settings.rOptions.rFiles do
    if AutoLoadLast then
      AutoLoadFile := FCatalogFile.CurrentFile;
  CanClose := FCatalogFile.Close;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1CompareItems(Sender: TObject; Item1, Item2: TElTreeItem; var res: Integer);
var
  Field : Integer;
  Movie1, Movie2: TMovie;
  ItemTmp: TElTreeItem;
begin
  if (ListView1.SortSection > -1) and (ListView1.SortSection < ListView1.HeaderSections.Count)
    and (ListView1.HeaderSections.Item[ListView1.SortSection].SortMode = hsmDescend) then
  begin
    ItemTmp := Item1;
    Item1 := Item2;
    Item2 := ItemTmp;
  end;
  if Item1 = Item2 then
    res := 0
  else
  if ((Item1.Data = nil) or (Item2.Data = nil)) then
  begin
    if (Item1.Data = nil) and (Item2.Data <> nil) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := -1 else res := 1
    else
    if (Item1.Data <> nil) and (Item2.Data = nil) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := 1 else res := -1
    else
    if StartsStr(strErrorParenthesis, Item1.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := 1 else res := -1
    else
    if StartsStr(strErrorParenthesis, Item2.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := -1 else res := 1
    else
    if StartsStr(Messages.Strings[msgGroupEmpty], Item1.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := 1 else res := -1
    else
    if StartsStr(Messages.Strings[msgGroupEmpty], Item2.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := -1 else res := 1
    else
    if StartsStr(Messages.Strings[msgGroupUnique], Item1.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := 1 else res := -1
    else
    if StartsStr(Messages.Strings[msgGroupUnique], Item2.Text) then
      if Settings.rOptions.rMovieList.GroupsAbove then res := -1 else res := 1
    else
    begin
      res := 0;
      if Settings.rOptions.rMovieList.SortGroupsByCount then
      begin
        res := Item2.Count - Item1.Count;
      end;
      if res = 0 then
      begin
        case GetFieldType(FGroupField) of
          ftInteger:  res := StrToIntTrunc(Item1.Text, 0) - StrToIntTrunc(Item2.Text, 0);
          ftReal:     res := CompareValue(StrToFloatTrunc(Item1.Text, 0), StrToFloatTrunc(Item2.Text, 0));
          ftDate:     res := Trunc(StrToDateDef(TextBefore(Item1.Text, ' ('), 0)) - Trunc(StrToDateDef(TextBefore(Item2.Text, ' ('), 0));
        else
          res := AnsiCompareText(Item1.Text, Item2.Text);
        end;
      end;
    end;
  end else
  begin
    if(FSortField < 0) then
    begin
      ItemTmp := Item1;
      Item1 := Item2;
      Item2 := ItemTmp;
    end;
    with ListView1 do
    begin
      Field := Abs(FSortField) - 1;
      Movie1 := Item1.Data;
      Movie2 := Item2.Data;
      case GetFieldType(Field) of
        ftInteger, ftReal, ftDate:
          res := Movie1.GetIntFieldValue(Field) - Movie2.GetIntFieldValue(Field);
        else
          res := AnsiCompareText(Movie1.GetFieldValue(Field), Movie2.GetFieldValue(Field));
        end;
      if res = 0 then
        res := AnsiCompareText(Movie1.GetFormattedTitle, Movie2.GetFormattedTitle);
      if res = 0 then
        res := Movie1.iNumber - Movie2.iNumber;
      if res = 0 then
        res := AnsiCompareStr(Format('%p', [Item1.Data]), Format('%p', [Item2.Data]));
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1Resize(Sender: TObject);
begin
  with ListView1 do
    if HeaderSections.Item[1].Visible then
      HeaderSections.Item[1].Width := ClientWidth - IfThen(HeaderSections.Item[0].Visible, HeaderSections.Item[0].Width) - IfThen(VertScrollBarVisible and Settings.rOptions.rMovieList.EnhancedScrollbars, VertScrollBarStyles.Width);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1ItemChecked(Sender: TObject;
  Item: TElTreeItem);
var
  i: Integer;
  _listItems: TObjectList;
begin
  if (Item <> nil) and (Item.Data <> nil) then
  begin
    TMovie(Item.Data).bChecked := Item.Checked;
    FCatalogFile.Modified := True;
    if ActionToolsGrid.Checked then
    begin
      ListView1.Items.BeginUpdate;
      _listItems := TMovie(Item.Data)._listItems;
      for i := 0 to _listItems.Count-1 do
      begin
        TElTreeItem(_listItems.Items[i]).SubItems.Strings[fieldChecked+2] := BoolToStr(Item.Checked, True);
        TElTreeItem(_listItems.Items[i]).Checked := Item.Checked;
      end;
      ListView1.Items.EndUpdate;
    end;
    //Item.Selected := not Item.Selected;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1AfterSelectionChange(Sender: TObject);
begin
  SaveCurrentItem;
  if (Sender <> ThumbsViewer) then
  begin
    if (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil) and
      (ListView1.Selected.Data <> nil) then
    begin
      if (ListView1.Selected.Parent <> nil) then
      begin
        if (ListView1.Selected.Parent.Index < ThumbsViewer.SmartGroups.Count) and
          (ListView1.Selected.Index < PSmartGroup(ThumbsViewer.SmartGroups[
          ListView1.Selected.Parent.Index]).Items.Count) then
        begin
          ThumbsViewer.Selection.Clear;
          ThumbsViewer.Selection.Add(MovieList.IndexOf(ListView1.Selected.Data));
          ThumbsViewer.IdxItem := ListView1.Selected.Index;
          ThumbsViewer.IdxGrp := ListView1.Selected.Parent.Index;
          ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, -1);
          ThumbsViewer.SetInView(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
          ThumbsViewer.Invalidate;
        end;
      end
      else
      begin
        if (ListView1.Selected.Index < ThumbsViewer.Items.Count) then
        begin
          ThumbsViewer.Selection.Clear;
          ThumbsViewer.Selection.Add(MovieList.IndexOf(ListView1.Selected.Data));
          ThumbsViewer.IdxItem := ListView1.Selected.Index;
          ThumbsViewer.IdxGrp := -1;
          ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
          ThumbsViewer.Invalidate;
        end;
      end;
    end
    else
    begin
      ThumbsViewer.Selection.Clear;
      ThumbsViewer.Invalidate;
    end;
  end;
  with ListView1 do
  begin
    if (Selected <> nil) and (Selected.Selected) then
    begin
      OpenItem(Selected);
    end;
    MovieSelected;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  i: Integer;
begin
  if (Key = Ord('A')) and (Shift = [ssCtrl]) then
  begin
    ListView1.SelectAll;
    MovieSelected;
  end
  else
  if (Key = VK_DELETE) and (Shift = []) then
    ActionMovieDelete.Execute
  else
  if ((Key = VK_MULTIPLY) and (Shift = [])) or ((Key = VK_ADD) and (Shift = [ssCtrl])) then
    with ListView1.Items do
      for i := 0 to Count-1 do
        Item[i].Expand(True)
  else
  if (Key = VK_SUBTRACT) and (Shift = [ssCtrl]) then
    with ListView1.Items do
      for i := 0 to Count-1 do
        Item[i].Collapse(True)
  else
  if (Key = Ord('T')) and (Shift = [ssCtrl]) then
  begin
    ListView1.FullExpand;
    ListView1EnsureVisiblePlus;
  end else
  if (Key = Ord('T')) and (Shift = [ssCtrl, ssShift]) then
  begin
    //ListView1.DeselectAll;
    //MovieSelected;
    ListView1.FullCollapse;
    ListView1EnsureVisiblePlus;
  end else
  if (Key = VK_SPACE) and (Shift = []) then
    with ListView1 do
      if ShowCheckboxes and (Selected <> nil) then
        with Selected do
          if Selected and (Data <> nil) then
          begin
            if Checked then
              ActionMovieUncheck.Execute
            else
              ActionMovieCheck.Execute;
          end;

end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DragDropFilesDrop(Sender: TObject; Pos: TPoint;
  Value: TStringList);
var
  FileName, ext: string;
  i: Integer;
  DoNotAsk, MultipleMoviecovers: Boolean;
  PicImportMethod: TPictureSelectOption;
begin
  MultipleMoviecovers := False;
  for i := 0 to Value.Count-1 do
  begin
    FileName := Value.Strings[i];
    ext := LowerCase(ExtractFileExt(FileName));

    // **** Catalog ****
    if IndexText(ext, extCatalog) <> -1 then
    begin
      FCatalogFile.Open(FileName);
    end else

    // **** Import something ****
    begin
      if (ListView1.Selected = nil) or (ListView1.Selected.Selected = False) then
        ActionMovieAddExecute(nil);
      if (ListView1.Selected <> nil) and (ListView1.Selected.Data <> nil) then
      begin

        // **** Picture ***
        if IndexText(ext, extImage) <> -1 then
        begin
          if ActionPicSelect.Enabled then
            with Settings.rOptions.rMovieInformation do
            begin
              PicImportMethod := TPictureSelectOption(Abs(rPicImport.GetInfoMethod));
              if rPicImport.GetInfoMethod <= 0 then
              begin
                with TPictureDragDropWin.Create(Application) do
                  try
                    ImageListHot.GetIcon(Ord(ICON_PICTUREOPEN), Icon);
                    if not Execute(PicImportMethod, DoNotAsk) then
                      Break; // next file
                    rPicImport.GetInfoMethod := Integer(PicImportMethod);
                    if not DoNotAsk then
                      rPicImport.GetInfoMethod := - rPicImport.GetInfoMethod;
                  finally
                    Release;
                  end;
              end;
              ThumbsViewerStop;
              ImportMoviePicture(FileName, PicImportMethod);
              LoadMoviePicture;
              ThumbsViewerStart;
            end;
        end else

        // **** Video ****
        if IndexText(ext, extVideo) <> -1 then
        begin
          try
            if GetInfoFromMediaFile(FileName, FrmMovie) then
            begin
              FrmMovie.Modified := True;
              FCatalogFile.Modified := True;
              ListView1UpdateItem;
            end;
          except
            on e: Exception do
              MessageWin.Execute(GetShortHint(ActionMovieImportFiles.Hint) + sLineBreak + sLineBreak + e.Message, mtError, [mbOk]);
          end;
        end  // if extVideo
        else

        // **** MovieCovers.com description files ****
        if SameText(ext, '.film') then
        begin
          FCatalogFile.Modified := True;
          if MultipleMoviecovers then
          begin
            ActionMovieAddExecute(nil);
            if (ListView1.Selected = nil) or (ListView1.Selected.Data = nil) then
              Exit;
          end else
            MultipleMoviecovers := True;
          with TStringList.Create do
            try
              FrmMovie.BeginUpdate;
              try
                LoadFromFile(FileName);
                try
                  FrmMovie.ETranslatedTitle.Text := Strings[0];
                  FrmMovie.EDirector.Text := Strings[1];
                  FrmMovie.EYear.Text := Strings[2];
                  FrmMovie.ECountry.Text := Strings[3];
                  FrmMovie.ECategory.Text := Strings[4];
                  FrmMovie.ELength.Value := StrToIntDef(TextBefore(UpperCase(Strings[5]), 'H'), 0) * 60 + StrToIntDef(TextAfter(UpperCase(Strings[5]), 'H'), 0);
                  FrmMovie.EActors.Text := Strings[6];
                  FrmMovie.EDescription.Text := Strings[7];
                  FrmMovie.EProducer.Text := Strings[8];
                  FrmMovie.EOriginalTitle.Text := Strings[9];
                except
                end;
                if FrmMovie.EOriginalTitle.Text = '' then
                begin
                  FrmMovie.EOriginalTitle.Text := FrmMovie.ETranslatedTitle.Text;
                  FrmMovie.ETranslatedTitle.Text := '';
                end;
              finally
                FrmMovie.EndUpdate;
              end;
              FrmMovie.Modified := True;
              FCatalogFile.Modified := True;
              ListView1UpdateItem;
            finally
              Free;
            end;
        end;
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SearchInternetClick(Sender: TObject);
var
  idx: Integer;
begin
  if Sender is TTBXItem then
  begin
    idx := (Sender as TTBXItem).Tag;
    with Settings.rOptions.rMovieInformation.SearchSites do
      if idx in [0..Count-1] then
        LaunchProg(Format(Values[Names[idx]], [TIdURI.ParamsEncode(FrmMovie.EOriginalTitle.Text)]));
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.PopupMovieListPopup(Sender: TObject);
var
  i: Integer;
begin
  with PopupMovieList.Items do
    for i := 0 to Count-1 do
      with Items[i] do
        Visible := Enabled;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.FrmMovieEURLButtonClick(Sender: TObject);
var
  FieldPos: TPoint;
begin
  with FrmMovie do
    FieldPos := ClientToScreen(Point(Left, Top));
  Inc(FieldPos.X, FrmMovie.EURL.Left);
  Inc(FieldPos.Y, FrmMovie.EURL.Top);
  PopupEURL.Popup(FieldPos.X + FrmMovie.EURL.Width, FieldPos.Y + FrmMovie.EURL.Height);
end;

{-------------------------------------------------------------------------------
   Picture panel
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMoviePictureShowExecute(Sender: TObject);
begin
  if ToolbarPictureWindow.Visible then
  begin
    ActionMoviePictureShow.Checked := False;
    ToolbarPictureWindow.Visible := False;
    SplitterBottomList.Visible := False;
  end else
  begin
    ActionMoviePictureShow.Checked := True;
    ToolbarPictureWindow.Visible := True;
    SplitterBottomList.Visible := not ToolbarPictureWindow.Floating;
    LoadMoviePicture;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ToolbarPictureWindowClose(Sender: TObject);
begin
  ActionMoviePictureShow.Checked := False;
  SplitterBottomList.Visible := False;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ToolbarPictureWindowDockChanged(Sender: TObject);
begin
  SplitterBottomList.Visible := ToolbarPictureWindow.Visible and not ToolbarPictureWindow.Floating;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionPicSelectExecute(Sender: TObject);
var
  ImportMethod: TPictureSelectOption;
begin
  with TPictureSelectionWin.Create(Self) do
    try
      InitialDir := Settings.rOptions.rFolders[fuPicture].Value;
      FileName := '';
      HelpContext := 1062;
      Options := DialogOpenOptions + [ofShowHelp];
      Title := Messages.Strings[msgPicOpen];
      Filter := DialogImageFilter;
      ImportMethod := TPictureSelectOption(Abs(Settings.rOptions.rMovieInformation.rPicImport.GetInfoMethod));
      if Execute(ImportMethod) then
      begin
        Settings.rOptions.rFolders[fuPicture].Value := ExtractFilePath(FileName);
        with Settings.rOptions.rMovieInformation do
          if rPicImport.GetInfoMethod > 0 then
            rPicImport.GetInfoMethod := Integer(ImportMethod)
          else
            rPicImport.GetInfoMethod := - Integer(ImportMethod);
        ThumbsViewerStop;
        ImportMoviePicture(FileName, ImportMethod);
        LoadMoviePicture;
        ThumbsViewerStart;
      end;
    finally
      Free;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionPicSaveAsExecute(Sender: TObject);
var
  Ext: string;
begin
  with ListView1 do
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data <> nil) then
          with TMovie(Data) do
            if strPicture <> '' then
              with TSaveDialog.Create(Self) do
                try
                  InitialDir := Settings.rOptions.rFolders[fuPicture].Value;
                  Options := DialogOpenOptions;
                  Title := Messages.Strings[msgPicSaveAs];
                  Ext := ExtractFileExt(strPicture);
                  case IndexText(ext, extImage) of
                    extPNG:
                      Filter := 'PNG (*.png)|*.png';
                    extJPG, extJPE, extJPEG:
                      Filter := 'JPEG (*.jpg, *.jpe, *.jpeg)|*.jpg;*.jpe;*.jpeg';
                    extGIF:
                      Filter := 'GIF (*.gif)|*.gif';
                  else
                    Filter := 'All files (*.*)|*.*';
                  end;
                  FileName := ValidateFileName(GetFormattedTitle);
                  if FileName = '' then
                    FileName := IntToStr(iNumber);
                  System.Delete(Ext, 1, 1);
                  DefaultExt := Ext;
                  if Execute then
                  begin
                    Settings.rOptions.rFolders[fuPicture].Value := ExtractFilePath(FileName);
                    if Picture <> nil then
                      Picture.SaveToFile(FileName)
                    else
                    begin
                      SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
                      CopyFile(PChar(strPicture), PChar(FileName), False);
                    end;
                  end;
                finally
                  Free;
                end
            else
            if strPicture <> '' then

end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionPicCopyExecute(Sender: TObject);
begin
  Clipboard.Assign(MoviePicture.Picture);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionPicDeleteExecute(Sender: TObject);
var
  res: TModalResult;
begin
  with ListView1 do
    if Selected <> nil then
      with Selected do
        if (Selected) and (Data <> nil) then
        begin
          ThumbsViewerStop;

          if(MovieSave = nil) then
          begin
            MovieSave := TMovie.Create;
            MovieSave.Assign(ListView1.Selected.Data);
          end;

          with TMovie(Data) do
          begin
            if Picture <> nil then
            begin
              if MessageWin.Execute(Messages.Strings[msgPicDelete], mtConfirmation, [mbOk, mbCancel]) = 1 then
              begin
                Picture.Free;
                Picture := nil;
                strPicture := '';
                FCatalogFile.Modified := True;
              end;
            end
            else
            if strPicture <> '' then
            begin
              if ExtractFilePath(strPicture) = '' then
              begin
                res := MessageWin.Execute(Messages.Strings[msgPicDelete], mtConfirmation, [mbOk, mbCancel]);
                if res = 2 then
                  res := 3;
              end
              else
              begin
                if FCatalogFile.CurrentFile <> '' then
                begin
                  SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
                  strPicture := ExpandFileName(strPicture);
                end;
                res := MessageWin.Execute(Format(Messages.Strings[msgPicDeleteLink], [strPicture]), mtConfirmation, [mbYes, mbNo, mbCancel]);
              end;
              if res in [1, 2] then
              begin
                if res = 1 then
                  DeleteFile(strPicture);
                strPicture := '';
                FCatalogFile.Modified := True;
              end;
            end;
            if (_thumb <> nil) then
              FreeAndNil(_thumb);
            _thumbError := 0;
          end;
          LoadMoviePicture;
          MovieSelected;
          ThumbsViewerStart;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionPicUndockExecute(Sender: TObject);
begin
  with ToolbarPictureWindow do
  begin
    Floating := not Floating;
    if not Floating then
    begin
      CurrentDock := DockBottomList;
      SplitterBottomList.Visible := True;
    end else
    begin
      SplitterBottomList.Visible := False;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.MoviePictureClick(Sender: TObject);
var
  CurMovie: TMovie;
begin
  if Assigned(ListView1.Selected) and Assigned(ListView1.Selected.Data) then
  begin
    PictureWin := TPictureWin.Create(Self);
    CurMovie := TMovie(ListView1.Selected.Data);
    try
      CurMovie.Lock;
      if (CurMovie.Picture = nil) then
      begin
        if CurMovie.strPicture = '' then
          Exit
        else
        begin
          if FCatalogFile.CurrentFile <> '' then
            SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
          PictureWin.Execute(CurMovie.GetFormattedTitle, ExpandFileName(CurMovie.strPicture));
        end;
      end
      else
        PictureWin.Execute(CurMovie.GetFormattedTitle, CurMovie.Picture, CurMovie.strPicture);
//        PictureWin.Execute(CurMovie.GetFormattedTitle, MoviePicture.Picture.Graphic);
    finally
      FreeAndNil(PictureWin);
      CurMovie.Unlock;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SplitterBottomListCanResize(Sender: TObject; var NewSize: Integer; var Accept: Boolean);
var
  d: TTBXDock;
  i: Integer;
begin
  d := ToolbarPictureWindow.CurrentDock as TTBXDock;
  if d = DockBottomList then
  begin
    i := d.ClientHeight - ToolbarPictureWindow.Height;
    (Sender as TSplitter).MinSize := i + 75;
    ToolbarPictureWindow.Height := NewSize - i;
  end
  else
  if d = DockRightInfo then
  begin
    i := d.ClientWidth - ToolbarPictureWindow.Width;
    (Sender as TSplitter).MinSize := i + 75;
    ToolbarPictureWindow.Width := NewSize - i;
  end;
end;

{-------------------------------------------------------------------------------
  Find Window
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFindFindnextExecute(Sender: TObject);
var
  Idx, StartIdx, SelectedField: Integer;
  WholeField, Finished: Boolean;
  Value: string;
begin
  with ListView1 do
    if Items.Count > 1 then
    begin
      WholeField := ActionFindWholefield.Checked;
      SelectedField := cbxField.ItemIndex - 1;
      Value := EValue.Text;
      if Selected = nil then
      begin
        StartIdx := 0;
        Idx := 0;
      end else
      begin
        StartIdx := Selected.AbsoluteIndex;
        Idx := StartIdx + 1;
      end;
      Finished := False;
      with Items do
        repeat
          if (Item[Idx] <> nil) and (Item[Idx].Data <> nil) then
            if TMovie(Item[Idx].Data).ContainsText(Value, SelectedField, WholeField) then
            begin
              Finished := True;
              DeselectAll;
              Selected := Item[Idx];
              Item[Idx].Selected := True;
              ListView1EnsureVisiblePlus;
              ListView1AfterSelectionChange(Sender);
            end;
          inc(Idx);
          if Idx >= Count then
            Idx := 0;
        until (StartIdx = Idx) or (Finished);
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFindDisplayExecute(Sender: TObject);
begin
  ActionFindFindnext.Enabled := not ActionFindDisplay.Checked;
  RefreshMovieList;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionFindWholefieldExecute(Sender: TObject);
begin
  if ActionFindDisplay.Checked then
    ActionFindDisplayExecute(ActionFindWholefield);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.EValueKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = Char(VK_RETURN) then
    ActionFindFindnext.Execute;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.cbxFieldChange(Sender: TObject);
begin
  if ActionFindDisplay.Checked then
    RefreshMovieList;
  HTMLViewerUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.EValueChange(Sender: TObject);
var
  OldCurPos: Integer;
begin
  if ActionFindDisplay.Checked then
  begin
    with EValue do
    begin
      OldCurPos := SelStart;
      RefreshMovieList;
      Application.ProcessMessages;
      SetFocus;
      SelLength := 0;
      SelStart := OldCurPos;
    end;
  end;
  HTMLViewerUpdate;
end;

{-------------------------------------------------------------------------------
  Toolbars
-------------------------------------------------------------------------------}

procedure TMainWindow.ToolbarMainClose(Sender: TObject);
begin
  ActionDisplayMainToolbar.Checked := false;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ToolbarPictureClose(Sender: TObject);
begin
  ActionDisplayPictureToolbar.Checked := false;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ToolbarFindVisibleChanged(Sender: TObject);
begin
  if ToolbarFind.Visible then
  begin
    ActionMovieFind.Checked := True;
    ActionFindDisplay.Checked := False;
    EValue.SetFocus;
  end else
  begin
    ActionMovieFind.Checked := False;
    with ActionFindDisplay do
      if Checked then
        Execute;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DockMainTopRequestDock(Sender: TObject;
  Bar: TTBCustomDockableWindow; var Accept: Boolean);
begin
  Accept := (Bar = ToolbarMain) or (Bar = ToolbarMenu) or (Bar = ToolbarPicture) or (Bar = ToolbarFind);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DockMainLeftRequestDock(Sender: TObject;
  Bar: TTBCustomDockableWindow; var Accept: Boolean);
begin
  Accept := (Bar = ToolbarMain) or (Bar = ToolbarMenu) or (Bar = ToolbarPicture);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DockMainBottomRequestDock(Sender: TObject;
  Bar: TTBCustomDockableWindow; var Accept: Boolean);
begin
  Accept := (Bar = ToolbarFind);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DockImageLeftRequestDock(Sender: TObject;
  Bar: TTBCustomDockableWindow; var Accept: Boolean);
begin
  Accept := (Bar = ToolbarPicture);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.DockBottomListRequestDock(Sender: TObject;
  Bar: TTBCustomDockableWindow; var Accept: Boolean);
begin
  Accept := (Bar = ToolbarPictureWindow);
end;

{-------------------------------------------------------------------------------
  Store & Restore selected items
-------------------------------------------------------------------------------}

procedure TMainWindow.StoreSelectedState;
var
//  DefCheck: Boolean;
  i: Integer;
//  PrevItem: TElTreeItem;
begin
  if MovieList <> nil then
    with MovieList do
      for i := 0 to Count-1 do
        with TMovie(Items[i]) do
        begin
          _bSelected := False;
          _bVisible := False;
          _selectedGroup := '';
          _selectedItem := nil;
        end;
  with ListView1 do
    for i := 0 to Items.Count-1 do
      if (Items[i].Data <> nil) then
        with TMovie(Items[i].Data) do
        begin
          if Items[i].Selected and (not _bSelected) then
          begin
            _bVisible := True;
            _bSelected := True;
            if Items[i].Parent <> nil then
              _selectedGroup := GetGroupNameWithoutCount(Items[i].Parent.Text);
          end;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.LoadSelectedState;
var
  i: Integer;
begin
  with ListView1 do
  begin
    DeselectAll;
    for i := 0 to Items.Count-1 do
      if Items[i].Data <> nil then
        with TMovie(Items[i].Data) do
        begin
          if _bSelected and ((_selectedItem = nil) or
            ((Items[i].Parent <> nil) and
            (AnsiCompareStr(GetGroupNameWithoutCount(Items[i].Parent.Text), _selectedGroup) = 0))) then
          begin
            if(_selectedItem <> nil) then
            begin
              TElTreeItem(_selectedItem).Selected := False;
              if (TElTreeItem(_selectedItem).Parent <> nil) and
                (not Settings.rOptions.rMovieList.GroupExpand) then
                TElTreeItem(_selectedItem).Parent.Expanded := False;
            end;
            _selectedItem := Items[i];
            ItemFocused := Items[i];
            Items[i].Selected := True;
            if (TElTreeItem(_selectedItem).Parent <> nil) then
              Items[i].Parent.Expanded := True;
          end;
        end;
    Selected := GetNextSelected(nil);
  end;
  ListView1AfterSelectionChange(ListView1);
end;

{-------------------------------------------------------------------------------
  Others
-------------------------------------------------------------------------------}

function TMainWindow.ExecuteAction(Action: TBasicAction): Boolean;
begin
  if (Action is THintAction) then
  begin
    StatusBar1.Panels[panelHint].Caption := THintAction(Action).Hint;
    Result := True;
  end
  else
    Result := inherited ExecuteAction(Action);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.OnAppHelp(Command: Word; Data: Integer; var CallHelp: Boolean): Boolean;
begin
  if Data <> Self.HelpContext then
    LaunchHelp(Data);
  CallHelp := False;
  Result := False;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionMovieRandomExecute(Sender: TObject);
var
  i, nb : Integer;
begin
  with ListView1 do
  begin
    if Items.Count > 0 then
    begin
      nb := 100;
      repeat
        i := Random(ListView1.Items.Count);
        nb := nb - 1;
      until (ListView1.Items.Item[i].Data <> nil) or (nb < 0);
      if (nb >= 0) then
      begin
        //ListView1UpdateItem; <-- Check in SaveCurrentItem
        if not Settings.rOptions.rMovieList.GroupExpand then
          FullCollapse;
        if (Selected <> Items.Item[i]) or (SelectedCount <> 1) then
        begin
          DeselectAll;
          ListView1.Selected := nil;
          ListView1.ItemFocused := nil;
          MultiSelect := False;
          Selected := Items.Item[i];
          MultiSelect := True;
          ListView1EnsureVisiblePlus;
          ListView1AfterSelectionChange(nil);
        end;
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1HeaderColumnClick(Sender: TObject;
  SectionIndex: Integer);
var
  Field: Integer;
  Descend: Boolean;
begin
  ListView1UpdateItem;
  Field := StrToInt(ListView1.HeaderSections.Item[SectionIndex].FieldName);
  Descend := FSortField < 0;
  if (Abs(FSortField) - 1) = Field then
    Descend := not Descend;
  SetSortField(Field, Descend);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionStretchListExecute(Sender: TObject);
begin
  ActionStretchList.Checked := not ActionStretchList.Checked;
  if ActionStretchList.Checked then
  begin
    PanelLeftOldWidth := PanelLeft.Width;
    PanelMovieInfos.Visible := False;
    SplitterMovieInfos.Visible := False;
    PanelLeft.Align := alClient;
    ListView1UpdateItem;
  end else
  begin
    PanelLeft.Align := alLeft;
    PanelLeft.Width := PanelLeftOldWidth;
    SplitterMovieInfos.Visible := True;
    PanelMovieInfos.Visible := True;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionDisplayHTMLExecute(Sender: TObject);
begin
  ActionDisplayHTML.Checked := not ActionDisplayHTML.Checked;
  HTMLViewer.Visible := ActionDisplayHTML.Checked;
  FrmMovie.Visible := not ActionDisplayHTML.Checked;
  HTMLViewerUpdate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ActionDisplayThumbnailsExecute(Sender: TObject);
begin
  ActionDisplayThumbnails.Checked := not ActionDisplayThumbnails.Checked;
  ThumbsPanel.Visible := ActionDisplayThumbnails.Checked;
  ListView1.Visible := not ActionDisplayThumbnails.Checked;
  if (ThumbsPanel.Visible) and (ThumbsViewer.Items.Count = 0) and
    (ThumbsViewer.SmartGroups.Count = 0) and (ListView1.Items.Count > 0) then
    ThumbsViewerStart;
  MovieSelected;
end;

procedure TMainWindow.HTMLViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if(HTMLViewer.CanFocus) then
    HTMLViewer.SetFocus;
end;

{-------------------------------------------------------------------------------
  HTML Viewer
-------------------------------------------------------------------------------}

procedure TMainWindow.PreGenerateHTMLTemplate;
var
  i, j, t, lastPos: integer;
  ATag : string;
  Found : Boolean;
  HTMLTemplate : string;
begin
  HTMLTemplatePreGenerated.Clear;
  HTMLTemplatePath := '';

  with Settings.rOptions.rFiles do
  begin
    if FileExists(HTMLTemplateFile) then
      HTMLTemplatePath := HTMLTemplateFile
    else if FileExists(strDirApp+'Templates\'+HTMLTemplateFile) then
      HTMLTemplatePath := strDirApp+'Templates\'+HTMLTemplateFile
    else if FileExists(strDirApp+HTMLTemplateFile) then
      HTMLTemplatePath := strDirApp+HTMLTemplateFile;

    if HTMLTemplatePath = '' then
    begin
      if HTMLTemplateFile = '' then
        HTMLTemplatePreGenerated.Add('No file template !')
      else
        HTMLTemplatePreGenerated.Add('"'+HTMLTemplateFile+'"'+' not found !');
      Exit;
    end;

    HTMLTemplatePreGenerated.LoadFromFile(HTMLTemplatePath);
    HTMLTemplate := HTMLTemplatePreGenerated.Text;
    HTMLTemplatePreGenerated.Clear;
  end;

  i := 1;
  lastPos := 1;
  while(HTMLTemplate[i] <> #0) do
  begin
    if (HTMLTemplate[i] = '$') and (HTMLTemplate[i+1] = '$') then
    begin
      Found := False;
      t := 0;
      while (t < strListTags.Count) and (not Found) do
      begin
        ATag := strListTags.Strings[t];
        j := 1;
        while (HTMLTemplate[i+j-1] <> #0) and (ATag[j] <> #0) do
        begin
          if (HTMLTemplate[i+j-1] <> ATag[j]) then
            break;
          if (ATag[j+1] = #0) then
          begin
            if (i <> lastPos) then
              HTMLTemplatePreGenerated.Add(Copy(HTMLTemplate, lastPos, i-lastPos));
            // Replace Labels
            if ATag = TAG_LABELNUMBER then HTMLTemplatePreGenerated.Add(strFields.Strings[0])
            else if ATag = TAG_LABELCHECKED then HTMLTemplatePreGenerated.Add(strFields.Strings[1])
            else if ATag = TAG_LABELMEDIA then HTMLTemplatePreGenerated.Add(strFields.Strings[2])
            else if ATag = TAG_LABELTYPE then HTMLTemplatePreGenerated.Add(strFields.Strings[3])
            else if ATag = TAG_LABELSOURCE then HTMLTemplatePreGenerated.Add(strFields.Strings[4])
            else if ATag = TAG_LABELDATEADD then HTMLTemplatePreGenerated.Add(strFields.Strings[5])
            else if ATag = TAG_LABELBORROWER then HTMLTemplatePreGenerated.Add(strFields.Strings[6])
            else if ATag = TAG_LABELRATING then HTMLTemplatePreGenerated.Add(strFields.Strings[7])
            else if ATag = TAG_LABELORIGINALTITLE then HTMLTemplatePreGenerated.Add(strFields.Strings[8])
            else if ATag = TAG_LABELTRANSLATEDTITLE then HTMLTemplatePreGenerated.Add(strFields.Strings[9])
            else if ATag = TAG_LABELFORMATTEDTITLE then HTMLTemplatePreGenerated.Add(strFields.Strings[10])
            else if ATag = TAG_LABELDIRECTOR then HTMLTemplatePreGenerated.Add(strFields.Strings[11])
            else if ATag = TAG_LABELPRODUCER then HTMLTemplatePreGenerated.Add(strFields.Strings[12])
            else if ATag = TAG_LABELCOUNTRY then HTMLTemplatePreGenerated.Add(strFields.Strings[13])
            else if ATag = TAG_LABELCATEGORY then HTMLTemplatePreGenerated.Add(strFields.Strings[14])
            else if ATag = TAG_LABELYEAR then HTMLTemplatePreGenerated.Add(strFields.Strings[15])
            else if ATag = TAG_LABELLENGTH then HTMLTemplatePreGenerated.Add(strFields.Strings[16])
            else if ATag = TAG_LABELACTORS then HTMLTemplatePreGenerated.Add(strFields.Strings[17])
            else if ATag = TAG_LABELURL then HTMLTemplatePreGenerated.Add(strFields.Strings[18])
            else if ATag = TAG_LABELDESCRIPTION then HTMLTemplatePreGenerated.Add(strFields.Strings[19])
            else if ATag = TAG_LABELCOMMENTS then HTMLTemplatePreGenerated.Add(strFields.Strings[20])
            else if ATag = TAG_LABELVIDEOFORMAT then HTMLTemplatePreGenerated.Add(strFields.Strings[21])
            else if ATag = TAG_LABELVIDEOBITRATE then HTMLTemplatePreGenerated.Add(strFields.Strings[22])
            else if ATag = TAG_LABELAUDIOFORMAT then HTMLTemplatePreGenerated.Add(strFields.Strings[23])
            else if ATag = TAG_LABELAUDIOBITRATE then HTMLTemplatePreGenerated.Add(strFields.Strings[24])
            else if ATag = TAG_LABELRESOLUTION then HTMLTemplatePreGenerated.Add(strFields.Strings[25])
            else if ATag = TAG_LABELFRAMERATE then HTMLTemplatePreGenerated.Add(strFields.Strings[26])
            else if ATag = TAG_LABELLANGUAGES then HTMLTemplatePreGenerated.Add(strFields.Strings[27])
            else if ATag = TAG_LABELSUBTITLES then HTMLTemplatePreGenerated.Add(strFields.Strings[28])
            else if ATag = TAG_LABELSIZE then HTMLTemplatePreGenerated.Add(strFields.Strings[29])
            else if ATag = TAG_LABELDISKS then HTMLTemplatePreGenerated.Add(strFields.Strings[30])
            else if ATag = TAG_LABELPICTURE then HTMLTemplatePreGenerated.Add(Messages.Strings[7])
            else if ATag = TAG_LABELAUDIOKBPS then HTMLTemplatePreGenerated.Add(FrmMovie.LAudioKbps.Caption)
            else if ATag = TAG_LABELVIDEOKBPS then HTMLTemplatePreGenerated.Add(FrmMovie.LVideoKbps.Caption)
            else if ATag = TAG_LABELFPS then HTMLTemplatePreGenerated.Add(FrmMovie.LFramerateFPS.Caption)
            else if ATag = TAG_LABELUNIT then HTMLTemplatePreGenerated.Add(FrmMovie.LSizeUnit.Caption)
            else HTMLTemplatePreGenerated.Add(ATag);
            lastPos := i+j;
            i := lastPos;
            Found := True;
          end;
          Inc(j)
        end;
        Inc(t);
      end;
    end;
    Inc(i);
  end;
  if (i <> lastPos) then
  begin
    HTMLTemplatePreGenerated.Add(Copy(HTMLTemplate, lastPos, i-lastPos));
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.GenerateHTMLSelectedMovie : string;
var
  LineBreak: string;
  i, n: integer;
  ImageFileName, ImageApprName: string;
  FHTMLIMGAttr, ATag: string;

  function GetV(Field: Integer; Text: string) : string;
  begin
    if (EValue.Text <> '') and ((cbxField.ItemIndex = 0) or (Field = (cbxField.ItemIndex - 1))) then
      Result := StringReplace(Text, EValue.Text, '<b><font color="#FF0000">'+EValue.Text+'</font></b>', [rfReplaceAll, rfIgnoreCase])
    else
      Result := Text;
  end;

begin
  Result := '';
  if (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil) and
    (ListView1.Selected.Data <> nil) then
  begin
    LineBreak := Settings.rOptions.rExport.Linebreak;
    ImageFileName := '*MoviePoster:';
    ImageApprName := '*StreamAppr:';
    FHTMLIMGAttr := '';
    with Settings.rOptions.rExport do
    begin
      if ForcePicSizeW >= 0 then
        FHTMLIMGAttr := Format(' width="%d"', [ForcePicSizeW]);
      if ForcePicSizeH >= 0 then
        FHTMLIMGAttr := Format('%s height="%d"', [FHTMLIMGAttr, ForcePicSizeH]);
    end;

    for n := 0 to HTMLTemplatePreGenerated.Count-1 do
    begin
      if (HTMLTemplatePreGenerated.Strings[n][1] <> '$') or (HTMLTemplatePreGenerated.Strings[n][2] <> '$') then
      begin
        Result := Result + HTMLTemplatePreGenerated.Strings[n];
        continue;
      end;
      with TMovie(ListView1.Selected.Data) do
      with FrmMovie do
      begin
        ATag := HTMLTemplatePreGenerated.Strings[n];
        // Replace Items
        if ATag = TAG_ITEMNUMBER then Result := Result + GetV(fieldNumber, GetFieldValue(fieldNumber))
        else if ATag = TAG_ITEMCHECKED then Result := Result + IfThen(bChecked, 'x', ' ')
        else if ATag = TAG_ITEMTYPE then Result := Result + GetV(fieldMediaType, EMediaType.Text)
        else if ATag = TAG_ITEMMEDIA then Result := Result + GetV(fieldMedia, EMedia.Text)
        else if ATag = TAG_ITEMSOURCE then Result := Result + GetV(fieldSource, ESource.Text)
        else if ATag = TAG_ITEMBORROWER then Result := Result + GetV(fieldBorrower, EBorrower.Text)
        else if ATag = TAG_ITEMDATEADD then
        begin
          if EDate.Checked then
            Result := Result + GetV(fieldDate, DateToStr(Trunc(EDate.DateTime)))
        end else if ATag = TAG_ITEMORIGINALTITLE then Result := Result + GetV(fieldOriginalTitle, EOriginalTitle.Text)
        else if ATag = TAG_ITEMTRANSLATEDTITLE then Result := Result + GetV(fieldTranslatedTitle, ETranslatedTitle.Text)
        else if ATag = TAG_ITEMFORMATTEDTITLE then Result := Result + GetV(fieldFormattedTitle, GetTitle)
        else if ATag = TAG_ITEMDIRECTOR then Result := Result + GetV(fieldDirector, EDirector.Text)
        else if ATag = TAG_ITEMPRODUCER then Result := Result + GetV(fieldProducer, EProducer.Text)
        else if ATag = TAG_ITEMCOUNTRY then Result := Result + GetV(fieldCountry, ECountry.Text)
        else if ATag = TAG_ITEMYEAR then Result := Result + GetV(fieldYear, EYear.Text)
        else if ATag = TAG_ITEMCATEGORY then Result := Result + GetV(fieldCategory, ECategory.Text)
        else if ATag = TAG_ITEMLENGTH then Result := Result + GetV(fieldLength, ELength.Text)
        else if ATag = TAG_ITEMACTORS then Result := Result + GetV(fieldActors, IfThen(LineBreak <> '', StringReplace(EActors.Text, #13#10, LineBreak, [rfReplaceAll]), EActors.Text))
        else if ATag = TAG_ITEMURL then Result := Result + GetV(fieldURL, EURL.Text)
        else if ATag = TAG_ITEMCOMMENTS then Result := Result + GetV(fieldComments, IfThen(LineBreak <> '', StringReplace(EComments.Text, #13#10, LineBreak, [rfReplaceAll]), EComments.Text))
        else if ATag = TAG_ITEMDESCRIPTION then Result := Result + GetV(fieldDescription, IfThen(LineBreak <> '', StringReplace(EDescription.Text, #13#10, LineBreak, [rfReplaceAll]), EDescription.Text))
        else if ATag = TAG_ITEMFORMAT then Result := Result + GetV(fieldVideoFormat, EVideoFormat.Text)
        else if ATag = TAG_ITEMVIDEOFORMAT then Result := Result + GetV(fieldVideoFormat, EVideoFormat.Text)
        else if ATag = TAG_ITEMAUDIOFORMAT then Result := Result + GetV(fieldAudioFormat, EAudioFormat.Text)
        else if ATag = TAG_ITEMVIDEOBITRATE then Result := Result + GetV(fieldVideoBitrate, EVideoBitrate.Text)
        else if ATag = TAG_ITEMAUDIOBITRATE then Result := Result + GetV(fieldAudioBitrate, EAudioBitrate.Text)
        else if ATag = TAG_ITEMRESOLUTION then Result := Result + GetV(fieldResolution, EResolution.Text)
        else if ATag = TAG_ITEMFRAMERATE then Result := Result + GetV(fieldFrameRate, EFramerate.Text)
        else if ATag = TAG_ITEMSIZE then Result := Result + GetV(fieldSize, ESize.Text)
        else if ATag = TAG_ITEMLANGUAGES then Result := Result + GetV(fieldLanguages, ELanguages.Text)
        else if ATag = TAG_ITEMSUBTITLES then Result := Result + GetV(fieldSubtitles, ESubtitles.Text)
        else if ATag = TAG_ITEMDISKS then Result := Result + GetV(fieldDisks, EDisks.Text)
				else if ATag = TAG_ITEMRATING then Result := Result + GetV(fieldRating, ERating.Text)
				else if ATag = TAG_ITEMPICTUREPATH then // Warning put before TAG_ITEMPICTURE !
					Result := Result + ImageFileName + IntToStr(Integer(@(ListView1.Selected.Data)))
				else if ATag = TAG_ITEMPICTURE then
					Result := Result + Format('<img src="%s" alt="poster%d"%s />', [ImageFileName + IntToStr(Integer(@(ListView1.Selected.Data))), iNumber, FHTMLIMGAttr])
        else if ATag = TAG_ITEMAPPR10 then
        begin
          if ERating.Text <> '' then
          begin
            i := Round(StrToFloat(ERating.Text));
					  Result := Result + Format('<img src="%s" alt="%d/10" />', [ImageApprName + IntToStr(Integer(@MStreamAppr10[i])), i]);
          end;
				end else if ATag = TAG_ITEMAPPRECIATION then
				begin
          if ERating.Text <> '' then
          begin
            i := 0;
            case Trunc(StrToFloatTrunc(ERating.Text)*10) of
               0..29:  begin i := 0; end;
              30..49:  begin i := 1; end;
              50..69:  begin i := 2; end;
              70..89:  begin i := 3; end;
              90..100: begin i := 4; end;
            end;
            Result := Result + Format('<img src="%s" alt="%d/4" />', [ImageApprName + IntToStr(Integer(@MStreamAppr[i])), i]);
          end;
				end else
          Result := Result + ATag;
      end; // with
    end; // for
  end; // if selected item <> nil
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.HTMLViewerImageRequest(Sender: TObject;
  const SRC: String; var Stream: TMemoryStream);
var
  PMovie: ^TMovie;
  PStream: ^TMemoryStream;
  ImageFileName: string;
begin
  if StartsStr('*MoviePoster:', SRC) then
  begin
    PMovie := Pointer(StrToInt(StringReplace(SRC, '*MoviePoster:', '', [])));
    with PMovie^ do
    begin
      if Picture <> nil then
        Stream := Picture
      else
      begin
        if strPicture = '' then
          ImageFileName := strDirApp + 'Templates\noposter.jpg'
        else
          ImageFileName := ExpandFileName(strPicture);
        if FileExists(ImageFileName) then
        begin
          MStream.LoadFromFile(ImageFileName);
          Stream := MStream;
        end;
      end;
    end;
  end else if StartsStr('*StreamAppr:', SRC) then
  begin
    PStream := Pointer(StrToInt(StringReplace(SRC, '*StreamAppr:', '', [])));
    Stream := PStream^;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.HTMLViewerImageClick(Sender, Obj: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  PMovie: ^TMovie;
begin
  if (Obj is TImageObj) and StartsStr('*MoviePoster:', TImageObj(Obj).Source) then
  begin
    PMovie := Pointer(StrToInt(StringReplace(TImageObj(Obj).Source, '*MoviePoster:', '', [])));
    PictureWin := TPictureWin.Create(Self);
    try
      PMovie^.Lock;
      if PMovie^.strPicture <> '' then
        if (PMovie^.Picture <> nil) then
          PictureWin.Execute(PMovie^.GetFormattedTitle, PMovie^.Picture, PMovie^.strPicture)
        else
        begin
          if FCatalogFile.CurrentFile <> '' then
            SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
          PictureWin.Execute(PMovie^.GetFormattedTitle, ExpandFileName(PMovie^.strPicture));
        end;
    finally
      FreeAndNil(PictureWin);
      PMovie^.Unlock;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.HTMLViewerKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if(Shift = [ssCtrl]) and (Key = Ord('C')) then
  begin
    HTMLViewer.CopyToClipboard;
  end
  else if(Shift = [ssCtrl]) and (Key = Ord('A')) then
  begin
    HTMLViewer.SelectAll;
  end;
end;

{-------------------------------------------------------------------------------
  TJPEGImageFix
-------------------------------------------------------------------------------}

procedure TJPEGImageFix.Draw(ACanvas: TCanvas; const Rect: TRect);
begin
  Bitmap.Canvas.Lock; // New
  try
    ACanvas.StretchDraw(Rect, Bitmap);
  finally
    Bitmap.Canvas.Unlock;
  end;
end;

{-------------------------------------------------------------------------------
  ThumbThread
-------------------------------------------------------------------------------}

constructor ThumbThread.Create(Thumbs: TrkSmartView; Items: TElTree; Movies: TMovieList);
begin
  ThumbsViewer := Thumbs;
  ListView1 := Items;
  MovieList := Movies;
  FreeOnTerminate := False;
  inherited Create(False);
  Priority := tpLower;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure ThumbThread.Execute;
var
  id, nb, nb10, i: Integer;
  Movie: TMovie;
  InView: Boolean;
  PtTmp: TPoint;
  List: TIntList;
begin
  inherited;
  if (ThumbsViewer.Items.Count <> 0) or (ThumbsViewer.SmartGroups.Count <> 0) then
  begin
    id := 0;
    nb := 0;
    nb10 := 0;

    i := 0;
    while (i < MovieList.Count) and (Terminated = False) do
    begin
      if(MovieList.Items[i]._thumb <> nil) or
        (MovieList.Items[i]._thumbError > 0) then
        Inc(nb);
      Inc(i);
    end;
    if (nb > 0) and (Terminated = False) then
    begin
      nb10 := Trunc(10*nb/MovieList.Count);
      PostMessage(MainWindow.Handle, CM_UpdateView, 0, nb10);
    end;

    while (nb < MovieList.Count) and (Terminated = False) do
    begin
      i := 0;
      InView := True;
      Movie := nil;
      ThumbsViewer.LockInView();
      while (i < ThumbsViewer.InView.Count) and
        ((Movie = nil) or (Movie._thumb <> nil) or (Movie._thumbError > 0)) do
      begin
        PtTmp := ThumbsViewer.InView.Points[i];
        if(PtTmp.X <> -1) then
          List := PSmartGroup(ThumbsViewer.SmartGroups.Items[PtTmp.X]).Items
        else
          List := ThumbsViewer.Items;
        Movie := MovieList.Items[List.Integers[PtTmp.Y]];
        Inc(i);
      end;
      ThumbsViewer.UnlockInView();

      if (Movie = nil) or (Movie._thumb <> nil) or (Movie._thumbError > 0) then
      begin
        InView := False;
        while (MovieList.Items[id]._thumb <> nil) or (MovieList.Items[id]._thumbError > 0) do
        begin
          Inc(id);
          if (id > MovieList.Count) then
            id := 0;
        end;
        Movie := MovieList.Items[id];
      end;

      Movie.Lock;
      MainWindow.ThumbsGetThumbnail(ThumbsViewer, Movie);
      Movie.Unlock;
      Inc(nb);

      if InView then
      begin
        nb10 := Trunc(10*nb/MovieList.Count);
        PostMessage(MainWindow.Handle, CM_UpdateView, 1, nb10)
      end
      else if(Trunc(10*nb/MovieList.Count) > nb10) then
      begin
        nb10 := Trunc(10*nb/MovieList.Count);
        PostMessage(MainWindow.Handle, CM_UpdateView, 0, nb10);
      end;
    end;
  end;
  PostMessage(MainWindow.Handle, CM_UpdateView, 1, 10);
  MainWindow.ThreadDone := True;
end;

{-------------------------------------------------------------------------------
  ThumbsViewer
-------------------------------------------------------------------------------}

procedure TMainWindow.CMUpdateView(var message: TMessage);
begin
  if message.WParam = 1 then
    ThumbsViewer.Invalidate;
  if message.LParam <> ProgressBar1.Position then
  begin
    ProgressBar1.Position := message.LParam;
    if ProgressBar1.Position = ProgressBar1.Max then
      ProgressBar1.Visible := False;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsGetThumbnail(Sender: TObject; Movie: TMovie);
var
  ThumbBmp, bmp: TBitmap;
  MS: TMemoryStream;
  ThumbSize: TPoint;
  newW, newH: Integer;
  sf: Integer;
  Ext: String;
  Path: String;
  ThumbGraphic: TGraphic;
  isLock: Boolean;

begin
  ThumbBmp := nil;
  bmp := nil;
  ThumbGraphic := nil;
  Path := '';
  isLock := False;

  if Movie.Picture = nil then
  begin
    if Movie.strPicture = '' then
      Path := strDirApp+'Templates\noposter.jpg'
    else
    begin
      if FCatalogFile.CurrentFile <> '' then
        SetCurrentDir(ExtractFilePath(FCatalogFile.CurrentFile));
      Path := Movie.strPicture;
    end;
    Ext := LowerCase(ExtractFileExt(ExtractFileName(Path)));
  end else
  begin
    Ext := Movie.strPicture;
  end;

  if (Ext = '.jpg') or (Ext = '.jpeg') then
    ThumbGraphic := ThumbJpeg
  else if (Ext = '.gif') then
    ThumbGraphic := ThumbGIF
  else if (Ext = '.png') then
    ThumbGraphic := ThumbPNG
  else
    Movie._thumbError := 1;

  if Movie._thumbError = 0 then
  begin
    if ThumbGraphic = ThumbJpeg then
    begin
      if (Movie.Picture = nil) then
        GetJPGSizeFromFile(Path, WI, HI)
      else
        GetJPGSize(Movie.Picture, WI, HI);
      if WI < 1 then
        WI := 1;
      if HI < 1 then
        HI := 1;
      sf := Trunc(Min(HI / 255, WI / 255));
      if sf < 0 then
        sf := 0;
      case sf of
        0..1: ThumbJpeg.Scale := jsFullSize;
        2..3: ThumbJpeg.Scale := jsHalf;
        4..7: ThumbJpeg.Scale := jsQuarter;
      else
        ThumbJpeg.Scale := jsEighth;
      end;
    end;

    try
      if (Movie.Picture = nil) then
        ThumbGraphic.LoadFromFile(Path)
      else
      begin
        Movie.Picture.Seek(0, soBeginning);
        ThumbGraphic.LoadFromStream(Movie.Picture);
      end;
      bmp := TBitmap.Create;
      bmp.PixelFormat := pf24Bit;
      bmp.Width := ThumbGraphic.Width;
      bmp.Height := ThumbGraphic.Height;
      bmp.Canvas.Lock;
      isLock := True;
      bmp.Canvas.Draw(0, 0, ThumbGraphic);

      try
        ThumbBmp := TBitmap.Create;
        ThumbBmp.Canvas.Lock;
        ThumbSize := CalcThumbSize(bmp.Width, bmp.Height,
          ThumbSizeW, ThumbSizeH);
        newW := ThumbSize.X;
        newH := ThumbSize.Y;
        if newW <= 0 then
          newW := 1;
        if newH <= 0 then
          newH := 1;
        ThumbBmp.PixelFormat := pf24Bit;
        ThumbBmp.Width := newW;
        ThumbBmp.Height := newH;
        MakeThumbNail(bmp, ThumbBmp);
      except
        ThumbBmp.Canvas.UnLock;
        FreeAndNil(ThumbBmp);
        Movie._thumbError := 3;
      end;

      bmp.Canvas.UnLock;
      bmp.Free;
    except
      Movie._thumbError := 2;
      if (bmp <> nil) then
      begin
        if (isLock) then
          bmp.Canvas.Unlock;
        bmp.Free;
      end;
    end;

    if ThumbBmp <> nil then
    begin
      MS := TMemoryStream.Create;
      MS.Seek(0, soBeginning);

      try
        ThumbJpeg.Assign(ThumbBmp);
        ThumbJpeg.Compress;
        ThumbJpeg.SaveToStream(MS);
        Movie._thumb := MS;
        Movie._thumbWidth := ThumbBmp.Width;
        Movie._thumbHeight := ThumbBmp.Height;
      except
        MS.Free;
        Movie._thumbError := 4;
      end;

      ThumbBmp.Canvas.UnLock;
      ThumbBmp.Free;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.StartThumbThread;
begin
  if ThumbThr <> nil then
    Exit;
  ThreadDone := False;
  ThumbThr := ThumbThread.Create(ThumbsViewer, ListView1, MovieList);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.StopThumbThread;
begin
  if ThumbThr <> nil then
  begin
    ThumbThr.Terminate;
    ThumbThr.WaitFor;
    ThumbThr.Free;
    ThumbThr := nil;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ClearThumbs;
begin
  ThumbsViewer.ClearSmartGroups;
  ThumbsViewer.Items.Clear;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ClearThumbsPool;
var
  i: Integer;
  Thumb: PCacheItem;
begin
  for i := ThumbsPool.Count - 1 downto 0 do
  begin
    Thumb := ThumbsPool[i];
    if Thumb.Bmp <> nil then
      Thumb.Bmp.Free;
    Dispose(Thumb);
  end;
  ThumbsPool.Clear;
  PoolSize := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerStop;
begin
  StopThumbThread;
  ClearThumbs;
  ClearThumbsPool;
  ThumbsViewer.Clear;
  ProgressBar1.Visible := False;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerStart;
var
  n, Grp, Idx: Integer;
  Item: TElTreeItem;
  Group: PSmartGroup;
  bSelect: Boolean;
begin
  ThumbsViewerStop;
  if not ThumbsPanel.Visible then
    Exit;

  Forms.Application.ProcessMessages;
  Group := nil;
  bSelect := (ListView1.SelectedCount = 1) and (ListView1.Selected <> nil);
  if (FGroupField <> -1) then
  begin
    Grp := -1;
    ThumbsViewer.Grouped := True;
    Item := ListView1.Items.GetFirstNode;
    while (Item <> nil) do
    begin
      if(Item.Data = nil) then
      begin
        Grp := ThumbsViewer.AddSmartGroup(Item.Text);
        Group := PSmartGroup(ThumbsViewer.SmartGroups.Items[Grp]);
        Group.Expanded := Item.Expanded;
      end else
      begin
        if (Item.Parent = nil) and ((Group = nil) or (Group.Caption <> '###')) then // Items not in group
        begin
          Grp := ThumbsViewer.AddSmartGroup('###');
          Group := PSmartGroup(ThumbsViewer.SmartGroups.Items[Grp]);
          Group.Expanded := True;
        end;
        n := MovieList.IndexOf(Item.Data);
        if (n <> -1) then
        begin
          Idx := Group.Items.Add(n);
          if (bSelect and Item.Selected) then
          begin
            ThumbsViewer.Selection.Add(n);
            ThumbsViewer.IdxGrp := Grp;
            ThumbsViewer.IdxItem := Idx;
          end;
        end;
      end;
      Item := Item.GetNext;
    end;
  end
  else
  begin
    ThumbsViewer.Grouped := False;
    Item := ListView1.Items.GetFirstNode;
    while (Item <> nil) do
    begin
      if Item.Data <> nil then
      begin
        n := MovieList.IndexOf(Item.Data);
        if (n <> -1) then
        begin
          Idx := ThumbsViewer.Items.Add(n);
          if (bSelect and Item.Selected) then
          begin
            ThumbsViewer.Selection.Add(n);
            ThumbsViewer.IdxGrp := -1;
            ThumbsViewer.IdxItem := Idx;
          end;
        end;
      end;
      Item := Item.GetNext;
    end;
  end;

  if(ListView1.Items.Count > 0) then
  begin
    ThumbsViewer.CalcView(True);
    SetThumbSize((ThumbsSizer.Position shl 4) - 1, False);
    if (FGroupField <> -1) then
    begin
      ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, -1);
      ThumbsViewer.SetInView(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
    end else
    begin
      ThumbsViewer.SetAtTop(-1, ThumbsViewer.IdxItem);
    end;
    ProgressBar1.Position := 0;
    ProgressBar1.Visible := True;
    if ThumbThr = nil then
    begin
      ThreadDone := False;
      ThumbThr := ThumbThread.Create(ThumbsViewer, ListView1, MovieList);
    end;
  end
  else
  begin
    ProgressBar1.Visible := False;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.GenCellColors;
begin
  cHot := $00FDDE99;
  cgHotStart := $00FDF5E6;
  cGHotEnd := $00FDFBF6;
  cSelected := $00FDCE99;
  cGSelectedStart := $00FCEFC4;
  cGSelectedEnd := $00FDF8EF;
  cShadeSelect := $00F8F3EA;
  cDisabled := $00D9D9D9;
  cGDisabledStart := $00EAE9E9;
  cGDisabledEnd := $00FCFBFB;
  cShadeDisabled := $00F6F5F5;
  cGHeaderStart := $00F9F9F9;
  cGHeaderEnd := $00FEFEFE;
  cGHeaderHotStart := $00FFEDBD;
  cGHeaderHotEnd := $00FFF7E3;
  cGHeaderSelStart := $00FCEABA;
  cGHeaderSelEnd := $00FCF4E0;
  cBackground := clWindow;
  cLineHighLight := $00FEFBF6;
  CellBkgColor := clWindow;
  CellBrdColor[False, False] := cDisabled;
  CellBrdColor[False, True] := cDisabled;
  CellBrdColor[True, False] := $00B5B5B5;
  CellBrdColor[True, True] := cSelected;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ItemPaintBasic(Canvas: TCanvas; R: TRect; State: TsvItemState);
var
  C: TColor;
begin
  Canvas.Brush.Style := bsClear;
  if (State = svSelected) or (State = svHot) then
  begin
    if (ThumbsViewer.Focused) and (State = svSelected) then
    begin
      Canvas.Pen.Color:= cSelected;
      GradientFill(Canvas, R, cGSelectedStart, cGSelectedEnd, tgTopBottom);
    end else
    if (State = svHot) then
    begin
      Canvas.Pen.Color:= cHot;
      GradientFill(Canvas, R, cGHotStart, cGHotEnd, tgTopBottom);
    end else
    begin
      Canvas.Pen.Color:= cDisabled;
      GradientFill(Canvas, R, cGDisabledStart, cGDisabledEnd, tgTopBottom);
    end;
    Canvas.Rectangle(R);
    if (ThumbsViewer.Focused) then C:= cShadeSelect else C:= cShadeDisabled;
    Canvas.Pen.Color:= C;
    Canvas.MoveTo(R.Left + 1, R.Top + 2);
    Canvas.LineTo(R.Left + 1, R.Bottom - 2);
    Canvas.LineTo(R.Right - 2, R.Bottom - 2);
    Canvas.LineTo(R.Right - 2, R.Top + 1);
    Canvas.Pen.Style:= psSolid;
    Canvas.Pixels[R.Left, R.Top]:= C;
    Canvas.Pixels[R.Left, R.Bottom - 1]:= C;
    Canvas.Pixels[R.Right - 1, R.Top]:= C;
    Canvas.Pixels[R.Right - 1, R.Bottom -1]:= C;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.SetThumbSize(Value: Integer; UpdateTrackbar: Boolean);
var
  w, h: Integer;
begin
  case Value of
    31..63: CellJpeg.Scale := jsQuarter;
    64..127: CellJpeg.Scale := jsHalf;
    128..255: CellJpeg.Scale := jsFullSize;
  else
    CellJpeg.Scale := jsEighth;
  end;
  w := Trunc(Value*0.75);
  h := Value;
  w := w + 10;
  if CellStyle = 0 then
    h := h + 10
  else
    h := h + 30;
  HSX := (w - 70) shr 1; // Help for calculating rating...
  ThumbsViewer.CellWidth := w;
  ThumbsViewer.CellHeight := h;
  CellScale := Value;
  if UpdateTrackbar then
  begin
    ThumbsSizer.OnChange := nil;
    ThumbsSizer.Position := Trunc((CellScale + 1) shr 4);
    ThumbsSizer.OnChange := ThumbsSizerChange;
  end;
  ThumbsViewer.CalcView(False);
  ThumbsViewer.Invalidate;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsSizerChange(Sender: TObject);
begin
  SetThumbSize((ThumbsSizer.Position shl 4) - 1, False);
  if (ThumbsViewer.Grouped) then
  begin
    ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, -1);
    ThumbsViewer.SetInView(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
  end
  else
    ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMainWindow.ThumbBmp(Idx: Integer): TBitmap;
var
  i, n, sf: Integer;
  p: PCacheItem;
  Movie: TMovie;
  Bmp, tmp: TBitmap;
  pt: TPoint;
  Oldest: TDateTime;
begin
  Result := nil;
  // if we have thumbs, see if we can find it...
  if ThumbsPool.Count > 0 then
  begin
    i := ThumbsPool.Count - 1;
    while (i >= 0) and (PCacheItem(ThumbsPool[i]).Idx <> Idx) do
      i := i - 1;
    if i <> -1 then
    begin
      p := ThumbsPool[i];
      if (p.Idx = Idx) then
      begin
        if (p.Scale = CellScale) then
        begin
          p.Age := Now;
          Result := p.Bmp
        end
        else
        begin
          PoolSize := PoolSize - p.Size;
          p.Bmp.Free;
          Dispose(p);
          ThumbsPool.Delete(i);
        end;
      end;
    end;
  end;
  // if we dont have a thumb, make one...
  if Result = nil then
  begin
    Movie := MovieList.Items[Idx];
    if Movie._thumb <> nil then
    begin
      sf := Trunc(Min(Movie._thumbWidth / Trunc(CellScale*0.75), Movie._thumbHeight / CellScale));
      if sf < 0 then
        sf := 0;
      case sf of
        0..1: CellJPEG.Scale := jsFullSize;
        2..3: CellJPEG.Scale := jsHalf;
        4..7: CellJPEG.Scale := jsQuarter;
      else
        CellJPEG.Scale := jsEighth;
      end;
      Movie._thumb.Seek(0, soBeginning);
      CellJPEG.LoadFromStream(Movie._thumb);

      Bmp := TBitmap.Create;
      Bmp.PixelFormat := pf24bit;
      pt := CalcThumbSize(CellJPEG.Width, CellJPEG.Height, Trunc(CellScale*0.75),
        CellScale);
      if pt.x <> CellJPEG.Width then
      begin
        tmp := TBitmap.Create;
        tmp.PixelFormat := pf24bit;
        tmp.Width := CellJPEG.Width;
        tmp.Height := CellJPEG.Height;
        tmp.Canvas.Draw(0, 0, CellJPEG);
        Bmp.Width := pt.x;
        Bmp.Height := pt.y;
        if (Bmp.Width > 4) and (Bmp.Height > 4) then
          BiReSample(tmp, Bmp, False)
        else
          bmp.Canvas.StretchDraw(Rect(0, 0, pt.X, pt.Y), tmp);
        tmp.Free;
      end
      else
      begin
        Bmp.Width := CellJPEG.Width;
        Bmp.Height := CellJPEG.Height;
        Bmp.Canvas.Draw(0, 0, CellJPEG);
      end;

      New(p);
      p.Idx := Idx;
      p.Size := (Bmp.Width * Bmp.Height) * 3;
      p.Age := Now;
      p.Scale := CellScale;
      p.Bmp := Bmp;
      ThumbsPool.Add(p);
      PoolSize := PoolSize + p.Size;
      Result := p.Bmp;
      // Purge thumbs if needed
      while (PoolSize > MaxPool) and (ThumbsPool.Count > 0) do
      begin
        Oldest := Now;
        n := 0;
        for i := 0 to ThumbsPool.Count - 1 do
        begin
          p := ThumbsPool[i];
          if p.Age <= Oldest then
          begin
            Oldest := p.Age;
            n := i;
          end;
        end;
        Assert(n >= 0);
        p := ThumbsPool[n];
        PoolSize := PoolSize - p.Size;
        p.Bmp.Free;
        Dispose(p);
        ThumbsPool.Delete(n);
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerCellPaint(Sender: TObject; Canvas: TCanvas;
  Cell: TRect; IdxA, Idx: Integer; State: TsvItemState);
var
  X, Y: Integer;
  F, S: Boolean;
  R: TRect;
  TW, TH: Integer;
  Txt: string;
  Movie: TMovie;
  pt: TPoint;
  LockOk, ThumbOk: Boolean;
begin
  Movie := MovieList.Items[Idx];
  LockOk := Movie.LockWait(0);
  if (LockOk) then
    ThumbOk := (Movie._thumb <> nil) and (Movie._thumbError = 0)
  else
    ThumbOk := False;
  ItemPaintBasic(Canvas, Cell, State);
  F := ThumbsViewer.Focused;
  S := State = svSelected;

  if ThumbOk then
    pt := CalcThumbSize(Movie._thumbWidth, Movie._thumbHeight, Trunc(CellScale*0.75),
      CellScale)
  else
    pt := CalcThumbSize(Trunc(CellScale*0.75), CellScale, Trunc(CellScale*0.75),
      CellScale);

  TW := pt.X;
  TH := pt.Y;
  X := Cell.Left + ((Cell.Right - (Cell.Left + TW)) shr 1);
  Y := Cell.Top + ((Cell.Bottom - (Cell.Top + TH)) shr 1);
  Y := Y - 10;
  R.Left := X;
  R.Top := Y;
  R.Right := X + TW;
  R.Bottom := Y + TH;

  if ThumbOk then
    Canvas.Draw(X, Y, ThumbBmp(idx))
  else if (Movie = nil) then
  begin
    txt := 'Group';
    DrawText(Canvas.Handle, PChar(txt), Length(txt), R, DT_END_ELLIPSIS or
    DT_SINGLELINE or DT_NOPREFIX or DT_CENTER or DT_VCENTER);
  end
  else if (Movie._thumbError = 0) then
  begin
    txt := 'Loading...';
    DrawText(Canvas.Handle, PChar(txt), Length(txt), R, DT_END_ELLIPSIS or
    DT_SINGLELINE or DT_NOPREFIX or DT_CENTER or DT_VCENTER);
  end
  else
  begin
    R.Top := R.Top - 10;
    R.Bottom := R.Bottom - 10;

    txt := 'No Poster';
    DrawText(Canvas.Handle, PChar(txt), Length(txt), R, DT_END_ELLIPSIS or
    DT_SINGLELINE or DT_NOPREFIX or DT_CENTER or DT_VCENTER);

    R.Top := R.Top + 20;
    R.Bottom := R.Bottom + 20;

    txt := 'Error '+ IntToStr(Movie._thumbError);
    DrawText(Canvas.Handle, PChar(txt), Length(txt), R, DT_END_ELLIPSIS or
    DT_SINGLELINE or DT_NOPREFIX or DT_CENTER or DT_VCENTER);

    R.Top := R.Top - 10;
    R.Bottom := R.Bottom - 10;
  end;

  Canvas.Pen.Color := CellBrdColor[F, S];
  InflateRect(R, 2, 2);
  Canvas.Rectangle(R);
  Canvas.Pen.Color := clWhite;
  InflateRect(R, -1, -1);
  Canvas.Rectangle(R);
  R := Cell;
  R.Top := R.Bottom - 20;
  txt := Movie.GetFormattedTitle;
  DrawText(Canvas.Handle, PChar(txt), Length(txt), R, DT_END_ELLIPSIS or
    DT_SINGLELINE or DT_NOPREFIX or DT_CENTER or DT_VCENTER);
  if (LockOk) then
    Movie.Unlock;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerSelecting(Sender: TObject; Count: Integer);
var
  TreeItem : TElTreeItem;
  IdxGrp, IdxItem : Integer;
begin
  IdxGrp := ThumbsViewer.IdxGrp;
  IdxItem := ThumbsViewer.IdxItem;
  if (IdxGrp <> -1) and (IdxItem = -1) then
  begin
    ListView1.Items[ThumbsViewer.GetAbsoluteIdx(IdxGrp, 0) + IdxGrp].Expanded :=
      PSmartGroup(ThumbsViewer.SmartGroups.Items[IdxGrp]).Expanded;
  end
  else if (ThumbsViewer.Selection.Count = 1) then
  begin
    if IdxItem <> -1 then
    begin
      if (ListView1.Items[ThumbsViewer.GetAbsoluteIdx(IdxGrp,
        IdxItem) + IdxGrp + 1] <> nil) then
      begin
        TreeItem := ListView1.Items[ThumbsViewer.GetAbsoluteIdx(IdxGrp,
          IdxItem) + IdxGrp + 1];
        if (ListView1.Selected <> TreeItem) or (ListView1.SelectedCount <> 1) then
        begin
          ListView1.DeselectAll;
          ListView1.Selected := nil;
          ListView1.ItemFocused := nil;
          ListView1.MultiSelect := False;
          ListView1.Selected := TreeItem;
          ListView1.MultiSelect := True;
          ListView1EnsureVisiblePlus;
          ListView1AfterSelectionChange(ThumbsViewer);
        end;
      end;
    end;
  end
  else
  begin
    ListView1.DeselectAll;
    ListView1AfterSelectionChange(ThumbsViewer);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if (ThumbsViewer.CanFocus) then
    ThumbsViewer.SetFocus;
  ListView1UpdateItem;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerDividerPaint(Sender: TObject;
  Canvas: TCanvas; Cell: TRect; Group: PSmartGroup; State: TsvItemState);
var
  R: TRect;
  txt: string;
  x: Integer;
  cFontColorSave: TColor;
begin
  cFontColorSave := Canvas.Font.Color;
  R := Cell;
  ItemPaintBasic(Canvas, Cell, State);
  if (Group.Expanded) then
  begin
    ListView1.MinusPicture.Transparent := True;
    Canvas.Draw(R.Left + 1, R.Top + 1, ListView1.MinusPicture);
  end else
  begin
    ListView1.PlusPicture.Transparent := True;
    Canvas.Draw(R.Left + 1, R.Top + 1, ListView1.PlusPicture);
  end;
  Canvas.Pen.Width := 1;
  Canvas.Brush.Style := bsClear;
  R.Left := R.Left + 20;
  Canvas.Font.Size := 8;
  Canvas.Font.Color := $00905422;
  txt := Group.Caption;
  DrawText(Canvas.Handle, PChar(txt), Length(txt), R,
    DT_END_ELLIPSIS or DT_SINGLELINE or DT_NOPREFIX or DT_VCENTER);
  R.Top := R.Top + 1;
  R.Bottom := R.Bottom + 1;
  x := Canvas.TextWidth(txt);
  R.Left := R.Left + x;
  Canvas.Pen.Color := $00F0F0F0;
  Canvas.MoveTo(x + 36, R.Top + 8);
  Canvas.LineTo(R.Right - 8, R.Top + 8);
  Canvas.Font.Color := cFontColorSave;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerResize(Sender: TObject);
begin
  if (ThumbsViewer.Grouped) then
  begin
    ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, -1);
    ThumbsViewer.SetInView(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
  end
  else
    ThumbsViewer.SetAtTop(ThumbsViewer.IdxGrp, ThumbsViewer.IdxItem);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerHeaderPaint(Sender: TObject;
  Canvas: TCanvas; Header: TRect; Offset, Active: Integer;
  State: TsvItemState; Columns: array of Integer);
begin
  //
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerHeaderClick(Sender: TObject;
  Column: Integer);
begin
  //
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1ItemCollapse(Sender: TObject;
  Item: TElTreeItem);
begin
  if (Item.Index < ThumbsViewer.SmartGroups.Count) then
  begin
    PSmartGroup(ThumbsViewer.SmartGroups.Items[Item.Index]).Expanded := False;
    ThumbsViewer.CalcView(False);
    ThumbsViewer.Invalidate;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ListView1ItemExpand(Sender: TObject;
  Item: TElTreeItem);
begin
  if (Item.Index < ThumbsViewer.SmartGroups.Count) then
  begin
    PSmartGroup(ThumbsViewer.SmartGroups.Items[Item.Index]).Expanded := True;
    ThumbsViewer.CalcView(False);
    ThumbsViewer.Invalidate;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if(Button = mbRight) then
  begin
    PopupMovieListPopup(ThumbsViewer);
    PopupMovieList.Popup(ThumbsViewer.ClientOrigin.X + X, ThumbsViewer.ClientOrigin.Y + Y);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerDblClick(Sender: TObject);
begin
  if (ActionStretchList.Checked = True) then
    ActionStretchListExecute(ThumbsViewer);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMainWindow.ThumbsViewerKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key <> Ord('A')) then
    ListView1KeyUp(ThumbsViewer, Key, Shift);
end;

end.
